@@ -1,8 +1,7 @@
 /*
- * Copyright 2010-2014, Sikuli.org, sikulix.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * modified RaiMan 2014
  */
 package org.sikuli.basics;
 
@@ -93,7 +92,7 @@ public class FileManager {
 			return -1;
 		}
 	}
-	
+
 	public static int isUrlUseabel(URL aURL) {
     HttpURLConnection conn = null;
 		try {
@@ -400,7 +399,7 @@ public class FileManager {
     }
     return fTempDir;
   }
-  
+
   public static int getRandomInt() {
     int rand = 1 + new Random().nextInt();
     return (rand < 0 ? rand * -1 : rand);
@@ -504,7 +503,7 @@ public class FileManager {
       }
     }
   }
-  
+
   public static File createTempFile(String suffix) {
     return createTempFile(suffix, null);
   }
@@ -549,7 +548,7 @@ public class FileManager {
     }
     return null;
   }
-  
+
   public static String saveTimedImage(BufferedImage img) {
     return saveTimedImage(img, ImagePath.getBundlePath(), null);
   }
@@ -557,7 +556,7 @@ public class FileManager {
   public static String saveTimedImage(BufferedImage img, String path) {
     return saveTimedImage(img, path, null);
   }
-  
+
   public static String saveTimedImage(BufferedImage img, String path, String name) {
     RunTime.pause(0.01f);
     File fImage = new File(path, String.format("%s-%d.png", name, new Date().getTime()));
@@ -622,14 +621,14 @@ public class FileManager {
         dest.close();
       }
     } catch (Exception ex) {
-      log(-1, "unzip: not possible: source:\n%s\ntarget:\n%s\n(%s)%s", 
+      log(-1, "unzip: not possible: source:\n%s\ntarget:\n%s\n(%s)%s",
           fpZip, fpTarget, entry.getName(), ex);
       return false;
     } finally {
-      try {      
+      try {
         inpZip.close();
       } catch (IOException ex) {
-        log(-1, "unzip: closing source:\n%s\n%s", fpZip, ex);    
+        log(-1, "unzip: closing source:\n%s\n%s", fpZip, ex);
       }
     }
     return true;
@@ -935,7 +934,12 @@ public class FileManager {
 				if (!jp.contains("!/")) {
 					jp += "!/";
 				}
-				String jpu = "jar:" + jp + "/" + fName;
+				String jpu = "jar:" + jp;
+				if (jp.endsWith("!/")) {
+				  jpu += fName;
+        } else {
+          jpu += "/" + fName;
+        }
 				return new URL(jpu);
 			}
       return new URL(path, slashify(fName, false));
@@ -1088,37 +1092,12 @@ public class FileManager {
       return null;
     }
     if (shouldReload) {
-      Image.reload(fpImage);
+      Image.reload(sImage);
     }
     return fpImage;
   }
 
   //TODO consolidate with FileManager and Settings
-  public static void zip(String path, String outZip) throws IOException, FileNotFoundException {
-    ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(outZip));
-    zipDir(path, zos);
-    zos.close();
-  }
-
-  private static void zipDir(String dir, ZipOutputStream zos) throws IOException {
-    File zipDir = new File(dir);
-    String[] dirList = zipDir.list();
-    byte[] readBuffer = new byte[1024];
-    int bytesIn;
-    for (int i = 0; i < dirList.length; i++) {
-      File f = new File(zipDir, dirList[i]);
-      if (f.isFile()) {
-        FileInputStream fis = new FileInputStream(f);
-        ZipEntry anEntry = new ZipEntry(f.getName());
-        zos.putNextEntry(anEntry);
-        while ((bytesIn = fis.read(readBuffer)) != -1) {
-          zos.write(readBuffer, 0, bytesIn);
-        }
-        fis.close();
-      }
-    }
-  }
-
 	public static void deleteNotUsedImages(String bundle, Set<String> usedImages) {
 		File scriptFolder = new File(bundle);
 		if (!scriptFolder.isDirectory()) {
@@ -1228,7 +1207,7 @@ public class FileManager {
       return "";
     }
   }
-  
+
   private static String doRreadFileToString(File fPath) throws IOException {
     StringBuilder result = new StringBuilder();
     BufferedReader reader = null;
