@@ -1,3 +1,9 @@
+/*
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
+ * Released under the MIT License.
+ *
+ */
+
 /**
  *
  */
@@ -80,9 +86,9 @@ public class Visual extends JComponent
   Color color;
   public static Color defColorFront = Color.MAGENTA;
   Color colorFront;
-  public static Color defColorBack = Color.RED;
+  public static Color defColorBack = Color.WHITE;
   Color colorBack;
-  public static Color defColorFrame = Color.WHITE;
+  public static Color defColorFrame = Color.BLACK;
   Color colorFrame;
   public static Color defColorText = Color.BLACK;
   Color colorText;
@@ -432,7 +438,7 @@ public class Visual extends JComponent
     this.offsety = offsety;
     offsetLocation(offsetx, offsety);
   }
-  
+
   // TODO: fix this
   float zoomLevel = 1.0f;
   public void setZoomLevel(float zoomLevel) {
@@ -467,7 +473,6 @@ public class Visual extends JComponent
         Debug.info("Updaing by offset:" + sklComp.autolayout);
         Debug.info("Updaing child:" + sklComp);
 
-
         if (sklComp.autolayout instanceof AutoLayoutByMovement) {
           ((AutoLayoutByMovement) sklComp.autolayout).x = bounds.x;
           ((AutoLayoutByMovement) sklComp.autolayout).y = bounds.y;
@@ -482,7 +487,6 @@ public class Visual extends JComponent
       }
     }
 
-
   }
   // this allows the component to be dragged to another location on the screen
   ComponentMover cm;
@@ -749,8 +753,8 @@ public class Visual extends JComponent
     }
   }
   //</editor-fold>
-  
-//<editor-fold defaultstate="collapsed" desc="layout">
+
+  //<editor-fold defaultstate="collapsed" desc="layout">
   public Visual left() {
     left(0);
     return this;
@@ -785,7 +789,7 @@ public class Visual extends JComponent
     }
     return this;
   }
-  
+
   public Visual below() {
     return below(0);
   }
@@ -798,6 +802,7 @@ public class Visual extends JComponent
     return this;
   }
 //</editor-fold>
+
   //<editor-fold defaultstate="collapsed" desc="AutoLayout">
   AutoLayout autolayout = null;
 
@@ -1255,4 +1260,4 @@ public class Visual extends JComponent
       throw new InternalError(e.toString());
     }
   }
-}
\ No newline at end of file
+}
