@@ -1,8 +1,7 @@
 /*
- * Copyright 2010-2014, Sikuli.org, sikulix.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * modified RaiMan 2013
  */
 package org.sikuli.natives;
 
@@ -16,20 +15,19 @@ public interface OSUtil {
   // Others: return 0 if succeeds, -1 if fails
 
   /**
-   * check if all needed native command libraries or packages are installed<br>
-   * and load needed libraries<br>
-   * terminates if something is wrong
+   * check if needed command libraries or packages are installed and working<br>
+   * if not ok, respective features will do nothing but issue error messages
    */
-  public void checkLibAvailability();
+  public void checkFeatureAvailability();
 
   public App.AppEntry getApp(int pid, String name);
 
   public Map<Integer, String[]> getApps(String name);
-  
+
   public int isRunning(App.AppEntry app);
 
 	public int open(String appName);
-  
+
   public int open(App.AppEntry app);
 
   // Windows: returns PID, 0 if fails
