@@ -1,3 +1,9 @@
+/*
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
+ * Released under the MIT License.
+ *
+ */
+
 /*
  * To change this template, choose Tools | Templates
  * and open the template in the editor.
@@ -21,13 +27,9 @@ public class SysUtil {
       return pkg + "MacUtil";
     } else if (os.startsWith("windows")) {
       return pkg + "WinUtil";
-    } else if (os.startsWith("linux")) {
-      return pkg + "LinuxUtil";
     } else {
-      System.out.println("[error] fatal: getOSUtilClass: your OS is not supported");
-      System.exit(1);
+      return pkg + "LinuxUtil";
     }
-    return null;
   }
 
   public static OSUtil getOSUtil() {
