@@ -1,8 +1,7 @@
 /*
- * Copyright 2010-2014, Sikuli.org, sikulix.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * modified RaiMan
  */
 package org.sikuli.script;
 
@@ -13,5 +12,6 @@ public enum FindFailedResponse{
    ABORT,
    PROMPT,
    SKIP,
-   RETRY
+   RETRY,
+   HANDLE
 };
