@@ -1,8 +1,7 @@
 /*
- * Copyright 2010-2014, Sikuli.org, sikulix.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * modified RaiMan
  */
 package org.sikuli.script;
 
@@ -31,6 +30,7 @@ public interface IRobot {
    void mouseMove(int x, int y);
    void mouseDown(int buttons);
    int mouseUp(int buttons);
+   void mouseReset();
    void clickStarts();
    void clickEnds();
    void smoothMove(Location dest);
@@ -49,4 +49,3 @@ public interface IRobot {
     */
    IScreen getScreen();
 }
-
