@@ -1,8 +1,7 @@
 /*
- * Copyright 2010-2014, Sikuli.org, sikulix.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * modified RaiMan
  */
 package org.sikuli.script;
 
@@ -14,7 +13,7 @@ import java.io.IOException;
  * function template for (alternative) Screen implementations
  */
 public interface IScreen {
-  
+
 	public IRobot getRobot();
 
 	public Rectangle getBounds();
@@ -35,6 +34,8 @@ public interface IScreen {
 
 	public int getID();
 
+	public String getIDString();
+
 	public ScreenImage getLastScreenImageFromScreen();
 
   public String getLastScreenImageFile(String path, String name) throws IOException;
@@ -44,12 +45,12 @@ public interface IScreen {
 	public int getW();
 
 	public int getY();
-	
+
 	public int getH();
 
 	public ScreenImage userCapture(String string);
 
 	public int getIdFromPoint(int srcx, int srcy);
-	
+
 	public String toStringShort();
 }
