@@ -1,8 +1,7 @@
 /*
- * Copyright 2010-2014, Sikuli.org, sikulix.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * modified RaiMan
  */
 package org.sikuli.script;
 
@@ -70,6 +69,7 @@ public class Image {
 
   private static String me = "Image: ";
   private static int lvl = 3;
+
   private static void log(int level, String message, Object... args) {
     Debug.logx(level, me + message, args);
   }
@@ -77,8 +77,8 @@ public class Image {
   private static List<Image> images = Collections.synchronizedList(new ArrayList<Image>());
   private static Map<URL, Image> imageFiles = Collections.synchronizedMap(new HashMap<URL, Image>());
   private static Map<String, URL> imageNames = Collections.synchronizedMap(new HashMap<String, URL>());
-  private static int KB = 1024;
-  private static int MB = KB * KB;
+  private static final int KB = 1024;
+  private static final int MB = KB * KB;
   private final static String isBImg = "__BufferedImage__";
 
   private static long currentMemory = 0;
@@ -132,18 +132,50 @@ public class Image {
   }
 
   public static void reload(String fpImage) {
-    URL uImage = FileManager.makeURL(fpImage);
+//    URL uImage = FileManager.makeURL(fpImage);
+    URL uImage = imageNames.get(fpImage);
     if (imageFiles.containsKey(uImage)) {
       Image image = imageFiles.get(uImage);
       int sizeOld = image.bsize;
       if (null != image.loadAgain()) {
         currentMemoryDownUp(sizeOld, image.bsize);
+        image.setLastSeen(null, 0);
       }
     }
   }
 
+  private static boolean ideShouldReload = false;
+  protected boolean wasRecaptured = false;
+  public static void setIDEshouldReload(Image img) {
+    ideShouldReload = true;
+    img.wasRecaptured = true;
+    img.lastSeen = null;
+  }
+
+  public static boolean getIDEshouldReload() {
+    boolean state = ideShouldReload;
+    ideShouldReload = false;
+    return state;
+  }
+
+  public boolean isRecaptured() {
+    boolean state = wasRecaptured;
+    wasRecaptured = false;
+    return state;
+  }
+
 //<editor-fold defaultstate="collapsed" desc="imageName">
   private String imageName = null;
+  private String imageNameGiven = null;
+  private boolean bHasIOException = false;
+
+  public boolean hasIOException() {
+    return bHasIOException;
+  }
+
+  public void setHasIOException(boolean state) {
+    bHasIOException = state;
+  }
 
   public String getImageName() {
     return imageName;
@@ -206,14 +238,14 @@ public class Image {
     return imageIsText;
   }
 
-  /**
-   * wether this image's name should be taken as text
-   * @param val
-   */
-  public Image setIsText(boolean val) {
+  private Image setIsText(boolean val) {
     imageIsText = val;
     return this;
   }
+
+  public String getText() {
+    return imageNameGiven;
+  }
 //</editor-fold>
 
 //<editor-fold defaultstate="collapsed" desc="isAbsolute">
@@ -236,11 +268,7 @@ public class Image {
 //<editor-fold defaultstate="collapsed" desc="isBundled">
   private boolean imageIsBundled = false;
 
-  /**
-   * mark this image as being contained in a bundle
-   * @param imageIsBundled
-   */
-  public Image setIsBundled(boolean imageIsBundled) {
+  private Image setIsBundled(boolean imageIsBundled) {
     this.imageIsBundled = imageIsBundled;
     return this;
   }
@@ -290,6 +318,7 @@ public class Image {
    * Set the value of waitAfter
    *
    * @param waitAfter new value of waitAfter
+   * @return the image
    */
   public Image setWaitAfter(int waitAfter) {
     this.waitAfter = waitAfter;
@@ -313,6 +342,7 @@ public class Image {
    * Set the value of offset
    *
    * @param offset new value of offset
+   * @return the image
    */
   public Image setOffset(Location offset) {
     this.offset = offset;
@@ -336,6 +366,7 @@ public class Image {
    * Set the value of similarity
    *
    * @param similarity new value of similarity
+   * @return the image
    */
   public Image setSimilarity(float similarity) {
     this.similarity = similarity;
@@ -371,6 +402,7 @@ public class Image {
    *
    * @param lastSeen Match
    * @param sim SimilarityScore
+   * @return the image
    */
   protected Image setLastSeen(Rectangle lastSeen, double sim) {
     this.lastSeen = lastSeen;
@@ -437,17 +469,16 @@ public class Image {
         if (!beSilent) {
           log(-1, "could not be loaded: %s", fileURL);
         }
+        bHasIOException = true;
 				fileURL = null;
         return null;
       }
       if (imageName != null) {
-        if (!imageFiles.containsKey(fileURL)) {
-          imageFiles.put(fileURL, this);
-          imageNames.put(imageName, fileURL);
-          bwidth = bImage.getWidth();
-          bheight = bImage.getHeight();
-          bsize = bImage.getData().getDataBuffer().getSize();
-        }
+        imageFiles.put(fileURL, this);
+        imageNames.put(imageName, fileURL);
+        bwidth = bImage.getWidth();
+        bheight = bImage.getHeight();
+        bsize = bImage.getData().getDataBuffer().getSize();
         log(lvl, "loaded: %s (%s)", imageName, fileURL);
         if (isCaching()) {
           int maxMemory = Settings.getImageCache() * MB;
@@ -476,6 +507,7 @@ public class Image {
         if (!beSilent) {
           log(-1, "could not be loaded again: %s", fileURL);
         }
+        bHasIOException = true;
 				imageFiles.remove(fileURL);
         return null;
       }
@@ -499,6 +531,7 @@ public class Image {
     imgTarget.setIsText(imageIsText);
     imgTarget.setIsBundled(imageIsBundled);
     imgTarget.setLastSeen(getLastSeen(), getLastSeenScore());
+    imgTarget.setHasIOException(hasIOException());
     if (isPattern()) {
       imgTarget.setSimilarity(similarity);
       imgTarget.setOffset(offset);
@@ -568,20 +601,20 @@ public class Image {
     return createImageValidate(img, true);
   }
 
-	/**
-	 * create new Image from
-	 * @param obj String, Pattern or other Image
-	 * @return Image
-	 */
-	protected static Image createFromObject(Object obj) {
-    if (obj instanceof String) {
-      return create((String) obj);
-    } else if (obj instanceof Image) {
-      return (Image) obj;
-    }  else if (obj instanceof Pattern) {
-      return Image.create((Pattern) obj);
+  protected static <PSI> Image getImageFromTarget(PSI target) {
+    if (target instanceof Pattern) {
+      return ((Pattern) target).getImage();
+    } else if (target instanceof String) {
+      Image img = get((String) target, true);
+      img = createImageValidate(img, true);
+      return img;
+    } else if (target instanceof Image) {
+      return (Image) target;
+    } else {
+        runTime.abortScripting("aborting script at:",
+                String.format("find, wait, exists: invalid parameter: %s", target));
     }
-    return new Image();
+    return null;
   }
 
   /**
@@ -603,6 +636,9 @@ public class Image {
     if (!img.isValid()) {
       if (Settings.OcrTextSearch) {
         img.setIsText(true);
+        if (Settings.isValidImageFilename(img.getName())) {
+          img.setIsText(false);
+        }
       } else {
         if (verbose) {
 					log(-1, "Image not valid, but TextSearch is switched off!");
@@ -628,7 +664,6 @@ public class Image {
     return imageAsFile;
   }
 
-
   /**
    * FOR INTERNAL USE: see get(String, boolean)
    *
@@ -651,44 +686,69 @@ public class Image {
     if (fName == null || fName.isEmpty()) {
       return null;
     }
-		fName = FileManager.slashify(fName, false);
     Image img = null;
-    URL fURL = null;
-    String fileName = Settings.getValidImageFilename(fName);
-    if (fileName.isEmpty()) {
-      log(-1, "not a valid image type: " + fName);
-      fileName = fName;
+    if (fName.startsWith("\t") && fName.endsWith("\t")) {
+      fName = fName.substring(1, fName.length() - 1);
+      img = new Image();
+      img.setIsText(true);
+    } else {
+      fName = FileManager.slashify(fName, false);
+      URL fURL = null;
+      String fileName = Settings.getValidImageFilename(fName);
+      if (fileName.isEmpty()) {
+        log(-1, "not a valid image type: " + fName);
+        fileName = fName;
+      }
+      File imgFile = new File(fileName);
+      if (imgFile.isAbsolute()) {
+        if (imgFile.exists()) {
+          fURL = FileManager.makeURL(fileName);
+        }
+      } else {
+        fURL = imageNames.get(fileName);
+        if (fURL == null) {
+          fURL = ImagePath.find(fileName);
+        }
+      }
+      if (fURL != null) {
+        img = imageFiles.get(fURL);
+        if (img != null && null == imageNames.get(img.imageName)) {
+          imageNames.put(img.imageName, fURL);
+        }
+      }
+      if (img == null) {
+        img = new Image(fileName, fURL, silent);
+        img.setIsAbsolute(imgFile.isAbsolute());
+      } else {
+        if (img.bimg != null) {
+          log(3, "reused: %s (%s)", img.imageName, img.fileURL);
+        } else {
+          if (Settings.getImageCache() > 0) {
+            img.load();
+          }
+        }
+      }
     }
-    File imgFile = new File(fileName);
+		img.imageNameGiven = fName;
+    return img;
+  }
+
+	protected static void set(Image img) {
+    URL fURL = null;
+    File imgFile = new File(img.getName());
     if (imgFile.isAbsolute()) {
       if (imgFile.exists()) {
-        fURL = FileManager.makeURL(fileName);
+        fURL = FileManager.makeURL(img.getName());
       }
     } else {
-      fURL = imageNames.get(fileName);
+      fURL = imageNames.get(img.getName());
       if (fURL == null) {
-        fURL = ImagePath.find(fileName);
+        fURL = ImagePath.find(img.getName());
       }
     }
     if (fURL != null) {
-      img = imageFiles.get(fURL);
-      if (img != null && null == imageNames.get(img.imageName)) {
-        imageNames.put(img.imageName, fURL);
-      }
+      img.init(img.getName(), fURL, true);
     }
-    if (img == null) {
-      img = new Image(fileName, fURL, silent);
-      img.setIsAbsolute(imgFile.isAbsolute());
-    } else {
-      if (img.bimg != null) {
-        log(3, "reused: %s (%s)", img.imageName, img.fileURL);
-      } else {
-        if (Settings.getImageCache() > 0) {
-          img.load();
-        }
-      }
-		}
-    return img;
   }
 
   protected static Image get(URL imgURL) {
@@ -793,8 +853,8 @@ public class Image {
   }
 
   protected static synchronized void purge(URL pathURL) {
-    List<Image> imagePurgeList = new ArrayList<Image>();
-    List<String> imageNamePurgeList = new ArrayList<String>();
+    List<Image> imagePurgeList = new ArrayList<>();
+    List<String> imageNamePurgeList = new ArrayList<>();
     URL imgURL;
     Image img;
     log(lvl, "purge: ImagePath: %s", pathURL.getPath());
@@ -827,6 +887,62 @@ public class Image {
     }
   }
 
+  public boolean isFile() {
+    if (isValid()) {
+      URL furl = getURL();
+      if ("file".equals(furl.getProtocol())) {
+        return true;
+      }
+    }
+    return false;
+  }
+
+  public File remove() {
+    URL furl = null;
+    if (isFile()) {
+      furl = getURL();
+      unCacheImage(furl);
+      return new File(furl.getPath());
+    }
+    return null;
+  }
+
+  public void delete() {
+    File fImg = remove();
+    if (null != fImg) FileManager.deleteFileOrFolder(fImg);
+  }
+
+  private String hasBackup = "";
+
+  protected boolean backup() {
+    if (isValid()) {
+      File fOrg = new File(fileURL.getPath());
+      File fBack = new File(fOrg.getParentFile(), "_BACKUP_" + fOrg.getName());
+      if (FileManager.xcopy(fOrg, fBack)) {
+        hasBackup = fBack.getPath();
+        log(lvl, "backup: %s created", fBack.getName());
+        return true;
+      }
+      log(-1, "backup: %s did not work", fBack.getName());
+    }
+    return false;
+  }
+
+  protected boolean restore() {
+    if (!hasBackup.isEmpty()) {
+      File fBack = new File(hasBackup);
+      File fOrg = new File(hasBackup.replace("_BACKUP_", ""));
+      if (FileManager.xcopy(fBack, fOrg)) {
+        log(lvl, "restore: %s restored", fOrg.getName());
+        FileManager.deleteFileOrFolder(fBack);
+        hasBackup = "";
+        return true;
+      }
+      log(-1, "restore: %s did not work", fBack.getName());
+    }
+    return false;
+  }
+
   /**
    * purge the given image file's in memory image data and remove it from cache.
    * @param imgFileName an absolute filename
@@ -845,10 +961,12 @@ public class Image {
     if (img == null) {
       return;
     }
-    img.bimg = null;
+    currentMemoryDown(img.bsize);
+    img.setBimg(null);
     images.remove(img);
   }
 
+
   /**
    * Print the current state of the cache
    */
@@ -903,6 +1021,9 @@ public class Image {
    * @return the name
    */
   public String getName() {
+    if (isText()) {
+      return imageNameGiven;
+    }
     return imageName;
   }
 
@@ -931,7 +1052,7 @@ public class Image {
    * @return true if lodable from file or is an in memory image
    */
   public boolean isValid() {
-    return fileURL != null || imageName.contains(isBImg);
+    return fileURL != null || getName().contains(isBImg);
   }
 
 	/**
@@ -968,6 +1089,10 @@ public class Image {
 	 * @return BufferedImage (might be null)
    */
   public BufferedImage get() {
+    return get(true);
+  }
+
+  protected BufferedImage get(boolean shouldLoad) {
     if (bimg != null) {
       if (fileURL == null) {
         log(lvl + 1, "getImage inMemory: %s", imageName);
@@ -976,7 +1101,11 @@ public class Image {
       }
       return bimg;
     } else {
-      return load();
+      if (shouldLoad) {
+        return load();
+      } else {
+        return null;
+      }
     }
   }
 
@@ -1247,7 +1376,7 @@ public class Image {
       byte[] data = convertBufferedImageToByteArray(img);
       org.sikuli.natives.Mat theMat = Vision.createMat(img.getHeight(), img.getWidth(), data);
       if (Settings.FindProfiling) {
-        Debug.logp("[FindProfiling] createCVMat [%d x %d]: %d msec", 
+        Debug.logp("[FindProfiling] createCVMat [%d x %d]: %d msec",
                 img.getWidth(), img.getHeight(), new Date().getTime() - theMatTime);
       }
       return theMat;
@@ -1319,7 +1448,7 @@ public class Image {
 
 	/**
 	 * find an image in another image
-	 * @param img
+	 * @param img image
 	 * @return a Match or null
 	 */
 	public Match find(Image img) {
@@ -1329,7 +1458,7 @@ public class Image {
 
 	/**
 	 * find all images in another image
-	 * @param img
+	 * @param img image
 	 * @return Match or null
 	 */
 	public Iterator<Match> findAll(Image img) {
@@ -1359,7 +1488,7 @@ public class Image {
 
 	/**
 	 * convenience method: get text from given image file
-	 * @param imgFile
+	 * @param imgFile image filename
 	 * @return the text or null
 	 */
 	public static String text(String imgFile) {
