@@ -1,8 +1,7 @@
 /*
- * Copyright 2010-2014, Sikuli.org, sikulix.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * added RaiMan 2013
  */
 package org.sikuli.script;
 
@@ -181,12 +180,26 @@ public class ImagePath {
         continue;
       }
       paths[i++] = p.getPath();
+      if (p.isFile()) {
+        paths[i - 1] = new File(p.getPath()).getAbsolutePath();
+      }
     }
     return paths;
   }
 
+  public static String getPath(int ix) {
+    PathEntry pe = imagePaths.get(0);
+    String path = null;
+    if (pe != null) {
+      path = pe.getPath();
+    }
+    return path;
+  }
+
   /**
    * print the list of path entries
+   *
+   * @param lvl debug level to use
    */
   public static void dump(int lvl) {
     log(lvl, "ImagePath has %d entries (valid %d)", imagePaths.size(), getCount());
@@ -355,7 +368,7 @@ public class ImagePath {
 		}
 		return true;
   }
-  
+
   public static boolean removeHTTP(String pathHTTP) {
 		try {
       String proto = "http://";
@@ -407,7 +420,7 @@ public class ImagePath {
     }
     return false;
   }
-  
+
   public static boolean addJar(String fpJar, String fpImage) {
     URL pathURL = null;
     if (new File(fpJar).exists()) {
@@ -605,7 +618,7 @@ public class ImagePath {
 			return null;
 		}
 		URL pathURL = null;
-    File fPath = new File(FileManager.normalizeAbsolute(fpMainPath, false)); 
+    File fPath = new File(FileManager.normalizeAbsolute(fpMainPath, false));
 		if (fPath.exists()) {
 			pathURL = FileManager.makeURL(fPath.getAbsolutePath());
 		} else {
