@@ -1,8 +1,7 @@
 /*
- * Copyright 2010-2014, Sikuli.org, sikulix.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * modified RaiMan
  */
 package org.sikuli.script;
 
@@ -15,6 +14,7 @@ import java.util.Map;
 import java.util.TreeMap;
 import org.sikuli.basics.Debug;
 import org.sikuli.basics.Settings;
+import org.sikuli.util.SysJNA;
 
 /**
  * this class implements an interface to the Java key system
@@ -267,7 +267,7 @@ public class Key {
 
   //<editor-fold defaultstate="collapsed" desc="KeyNames to UniCode (used in type() with Key.XXX)">
   public static final String SPACE = " ";
-  public static final String ENTER = "\n";
+  public static final String ENTER = "\r";
   public static final String BACKSPACE = "\b";
   public static final String TAB = "\t";
   public static final String ESC = "\u001b";
@@ -380,8 +380,8 @@ public class Key {
   public static final char C_DECIMAL = '\ue040'; // VK_DECIMAL
   public static final String CONTEXT = "\ue041";
   public static final char C_CONTEXT = '\ue041'; // VK_CONTEXT_MENU
-  public static final String NEXT = "\ue043";
-  public static final char C_NEXT = '\ue043'; // VK_CONTEXT_MENU
+  public static final String NEXT = "\ue044";
+  public static final char C_NEXT = '\ue044'; // VK_CONTEXT_MENU
 
   public static final char cMax = '\ue050';
   public static final char cMin = '\ue000';
@@ -475,7 +475,7 @@ public class Key {
       return keyTexts.get(key).intValue();
     }
   }
-  
+
   public static void dump() {
     Map<Integer, String> namesVK = new HashMap<Integer, String>();
     for (int i = 0; i < keyVK.length; i += 2) {
@@ -490,7 +490,7 @@ public class Key {
       if (keyN < 1) {
         continue;
       }
-      System.out.println(String.format("%s = %d (%s)", key, keyN, namesVK.get(keyN)));      
+      System.out.println(String.format("%s = %d (%s)", key, keyN, namesVK.get(keyN)));
     }
   }
   //</editor-fold>
@@ -689,7 +689,7 @@ public class Key {
       case Key.C_NEXT:   return new int[]{-KeyEvent.VK_TAB};
 
       default:
-        throw new IllegalArgumentException("Cannot convert character " + key);
+        throw new IllegalArgumentException("Key: Not supported character: " + key);
     }
   }
 
@@ -803,14 +803,20 @@ public class Key {
    * @return true/false
    */
   public static boolean isLockOn(char key) {
-    Toolkit tk = Toolkit.getDefaultToolkit();
+//    Toolkit tk = Toolkit.getDefaultToolkit();
+//        return tk.getLockingKeyState(KeyEvent.VK_SCROLL_LOCK);
+//        return tk.getLockingKeyState(KeyEvent.VK_CAPS_LOCK);
+//        return tk.getLockingKeyState(KeyEvent.VK_NUM_LOCK);
+    if (!RunTime.get().runningWindows) {
+      return false;
+    }
     switch (key) {
       case '\ue025':
-        return tk.getLockingKeyState(KeyEvent.VK_SCROLL_LOCK);
+        return SysJNA.WinUser32.isScrollLockOn();
       case '\ue027':
-        return tk.getLockingKeyState(KeyEvent.VK_CAPS_LOCK);
+        return SysJNA.WinUser32.isCapsLockOn();
       case '\ue03B':
-        return tk.getLockingKeyState(KeyEvent.VK_NUM_LOCK);
+        return SysJNA.WinUser32.isNumLockOn();
       default:
         return false;
     }
@@ -988,6 +994,13 @@ public class Key {
 //		}
 //	}
 
+  /**
+   * INTERNAL USE ONLY
+   *
+   * @param code keycode
+   * @param mod modifier
+   * @return readable key text
+   */
   public static String convertKeyToText(int code, int mod) {
     String txtMod = KeyEvent.getKeyModifiersText(mod);
     String txtCode = KeyEvent.getKeyText(code);
