@@ -1,14 +1,11 @@
+/*
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
+ * Released under the MIT License.
+ *
+ */
+
 package org.sikuli.script;
 
-import java.io.File;
-import java.io.FileReader;
-import java.lang.reflect.Method;
-import java.net.URL;
-import java.util.Arrays;
-import java.util.HashMap;
-import java.util.Map;
-import javax.script.ScriptEngine;
-import javax.script.ScriptEngineManager;
 import org.apache.commons.cli.CommandLine;
 import org.sikuli.basics.Debug;
 import org.sikuli.basics.FileManager;
@@ -17,6 +14,20 @@ import org.sikuli.util.CommandArgs;
 import org.sikuli.util.CommandArgsEnum;
 import org.sikuli.util.JythonHelper;
 
+import javax.script.ScriptEngine;
+import javax.script.ScriptEngineManager;
+import java.io.File;
+import java.io.FileReader;
+import java.lang.reflect.Method;
+import java.net.URL;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.Map;
+
+/**
+ * INTERNAL USE --- NOT official API<br>
+ *   not in version 2
+ */
 public class Runner {
 
   static final String me = "Runner: ";
@@ -243,7 +254,7 @@ public class Runner {
         return false;
       }
       pyRunner.exec("# -*- coding: utf-8 -*- ");
-      pyRunner.exec("import org.sikuli.basics.SikulixForJython");
+      pyRunner.exec("import org.sikuli.script.SikulixForJython");
       pyRunner.exec("from sikuli import *");
     }
     return true;
@@ -448,8 +459,8 @@ public class Runner {
       }
     }
     Object[] vars = new Object[]{givenScriptHost, givenScriptFolder, givenScriptName,
-      givenScriptScript, givenScriptType, givenScriptScriptType,
-      uGivenScript, uGivenScriptFile, givenScriptExists, scriptProject, uScriptProject};
+            givenScriptScript, givenScriptType, givenScriptScriptType,
+            uGivenScript, uGivenScriptFile, givenScriptExists, scriptProject, uScriptProject};
     return vars;
   }
 
@@ -466,6 +477,9 @@ public class Runner {
   }
 
   public static int runrobot(String code) {
+    if (!JythonHelper.get().prepareRobot()) {
+      return -1;
+    }
     File script = new File(ImagePath.getBundlePath());
     File fRobotWork = new File(script.getAbsolutePath() + ".robot");
     FileManager.deleteFileOrFolder(fRobotWork);
@@ -474,64 +488,57 @@ public class Runner {
     File fPyCode = new File(script, sName + ".py");
     String pyCode = FileManager.readFileToString(fPyCode);
     int prefix = pyCode.indexOf("\"\"\")");
-    if (prefix < 0) {
-      log(-1, "runRobot: invalid robot text structure");
-      return -1;
-    }
-    pyCode = pyCode.substring(prefix + 4).trim();
-    int refLib = code.indexOf("./inline/");
-    String sLib = "";
-    File fInline = null;
-    String fpInline = "";
-    if (!pyCode.isEmpty()) {
-      if (refLib < 0) {
-        log(-1, "runRobot: invalid robot text structure");
-        return -1;
-      }
-      sLib = code.substring(refLib + 9);
-      sLib = sLib.substring(0, sLib.indexOf("\n")).trim();
-    } else {
-      if (refLib > -1) {
-        log(-1, "runRobot: invalid robot text structure");
-        return -1;
-      }
-      refLib = code.indexOf("./jar/");
-      if (refLib > -1) {
-        sLib = code.substring(refLib + 6);
-        sLib = sLib.substring(0, sLib.indexOf("\n")).trim();
-        fInline = new File (sLib + ".jar");
-        if (!fInline.isAbsolute()) {
-          fInline = new File(fRobotWork, sLib + ".jar");
+    if (prefix > 0) {
+      pyCode = pyCode.substring(prefix + 4).trim();
+      int refLib = code.indexOf("./inline/");
+      String inlineLib = "";
+      File fInline = null;
+      String fpInline = "";
+      // Keyword implementations are inline
+      if (!pyCode.isEmpty()) {
+        if (refLib < 0) {
+          log(-1, "runRobot: inline code ignored - no ./inline/");
+        }
+        inlineLib = code.substring(refLib + 9);
+        inlineLib = inlineLib.substring(0, inlineLib.indexOf("\n")).trim();
+        fInline = new File(fRobotWork, inlineLib + ".py");
+        pyCode = "from sikuli import *\n" + pyCode;
+        FileManager.writeStringToFile(pyCode, fInline);
+        fpInline = FileManager.slashify(fInline.getAbsolutePath(), false);
+        code = code.replace("./inline/" + inlineLib, fpInline);
+      } else {
+        if (refLib > -1) {
+          log(-1, "runRobot: having ./inline/, but no inline code found");
+          return -1;
         }
-        runTime.addToClasspath(fInline.getAbsolutePath());
-        sLib = "";
       }
     }
-    if (!sLib.isEmpty()) {
-      fInline = new File(fRobotWork, sLib + ".py");      
-      FileManager.writeStringToFile(pyCode, fInline);
-      fpInline = FileManager.slashify(fInline.getAbsolutePath(), false);
-      code = code.replace("./inline/" + sLib, fpInline );
-    }
     File fRobot = new File(fRobotWork, sName + ".robot");
     FileManager.writeStringToFile(code, fRobot);
     if (!initpy()) {
       log(-1, "Running Python scripts:init failed");
       return -999;
     }
-//    String jarR4S = runTime.isOnClasspath("robot4sikulix");
-//    if (null != jarR4S) {
-//      pyRunner.addSysPath(jarR4S);
-//    }
-    pyRunner.exec("from sikuli import *;");
-    pyRunner.exec("import robot.run;");
-    pyRunner.exec(String.format(
-                  "robot.run(\"%s\", "
-                  + "outputdir=\"%s\""
-                  + ");", fRobot, fRobotWork));
+    pyRunner.exec("from sikuli import *; from threading import currentThread; currentThread().name = \"MainThread\"");
+    //pyRunner.exec("import robot.run;");
+    String robotCmd = String.format(
+            "ret = robot.run(\"%s\", "
+                    + "outputdir=\"%s\")", fRobot, fRobotWork);
+    File fReport = new File(fRobotWork, "report.html");
+    String urlReport = fReport.getAbsolutePath();
+    if (RunTime.get().runningWindows) {
+      robotCmd = robotCmd.replaceAll("\\\\", "\\\\\\\\");
+      urlReport = "/" + urlReport.replaceAll("\\\\", "/");
+    }
+    pyRunner.exec(robotCmd + "; print \"robot.run returned:\", ret; " +
+            String.format("print \"robot.run output is here:\\n%s\";",
+            fRobotWork.getAbsolutePath().replaceAll("\\\\", "\\\\\\\\")));
+    if (new File(fRobotWork, "report.html").exists()) {
+      App.openLink("file:" + urlReport);
+    }
     return 0;
   }
-  
+
   //<editor-fold defaultstate="collapsed" desc="robot run options">
 //  -N --name name           Set the name of the top level test suite. Underscores
 //                          in the name are converted to spaces. Default name is
@@ -845,13 +852,13 @@ public class Runner {
     File aFile = FileManager.createTempFile("ps1");
     FileManager.writeStringToFile(givenScriptScript, aFile);
     String[] psDirect = new String[]{
-      "powershell.exe", "-ExecutionPolicy", "UnRestricted",
-      "-NonInteractive", "-NoLogo", "-NoProfile", "-WindowStyle", "Hidden",
-      "-File", aFile.getAbsolutePath()
+            "powershell.exe", "-ExecutionPolicy", "UnRestricted",
+            "-NonInteractive", "-NoLogo", "-NoProfile", "-WindowStyle", "Hidden",
+            "-File", aFile.getAbsolutePath()
     };
     String[] psCmdType = new String[]{
-      "cmd.exe", "/S", "/C",
-      "type " + aFile.getAbsolutePath() + " | powershell -noprofile -"
+            "cmd.exe", "/S", "/C",
+            "type " + aFile.getAbsolutePath() + " | powershell -noprofile -"
     };
     String retVal = runTime.runcmd(psCmdType);
     String[] parts = retVal.split("\\s");
@@ -1005,7 +1012,7 @@ public class Runner {
       return 0;
     }
 
-//    static File scriptProject = null;
+    //    static File scriptProject = null;
 //    static URL uScriptProject = null;
     RunTime runTime = RunTime.get();
     boolean asTest = false;
