@@ -1,21 +1,17 @@
 /*
- * Copyright 2010-2014, Sikuli.org, Sikulix.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * modified RaiMan
  */
 package org.sikuli.script;
 
-import org.sikuli.util.ScreenHighlighter;
-import org.sikuli.util.OverlayCapturePrompt;
-import org.sikuli.util.EventSubject;
-import org.sikuli.util.EventObserver;
-import org.sikuli.basics.Settings;
-import org.sikuli.basics.Debug;
-import java.awt.AWTException;
-import java.awt.Rectangle;
-import java.awt.Robot;
+import java.awt.*;
 import java.util.Date;
+import org.sikuli.basics.Debug;
+import org.sikuli.basics.Settings;
+import org.sikuli.util.EventObserver;
+import org.sikuli.util.OverlayCapturePrompt;
+import org.sikuli.util.ScreenHighlighter;
 
 /**
  * A screen represents a physical monitor with its coordinates and size according to the global
@@ -27,7 +23,7 @@ import java.util.Date;
  * the region of the screen more than once, you have to create new ones based on the screen.
  * <br>The so called primary screen is the one with top left (0,0) and has id 0.
  */
-public class Screen extends Region implements EventObserver, IScreen {
+public class Screen extends Region implements IScreen {
 
   static RunTime runTime = RunTime.get();
 
@@ -49,7 +45,25 @@ public class Screen extends Region implements EventObserver, IScreen {
   protected boolean waitPrompt;
   protected OverlayCapturePrompt prompt;
   private final static String promptMsg = "Select a region on the screen";
+  public static boolean ignorePrimaryAtCapture = false;
   private ScreenImage lastScreenImage = null;
+  private static boolean isActiveCapturePrompt = false;
+  private static EventObserver captureObserver = null;
+
+  private static synchronized boolean setActiveCapturePrompt() {
+    if (isActiveCapturePrompt) {
+      return false;
+    }
+    Debug.log(3, "TRACE: Screen: setActiveCapturePrompt");
+    isActiveCapturePrompt = true;
+    return true;
+  }
+
+  private static synchronized void resetActiveCapturePrompt() {
+    Debug.log(3, "TRACE: Screen: resetActiveCapturePrompt");
+    isActiveCapturePrompt = false;
+    captureObserver = null;
+  }
 
   //<editor-fold defaultstate="collapsed" desc="Initialization">
 
@@ -73,67 +87,81 @@ public class Screen extends Region implements EventObserver, IScreen {
     }
     log(lvl+1, "initScreens: entry");
     primaryScreen = 0;
-    globalRobot = getMouseRobot();
-    screens = new Screen[runTime.nMonitors];
-    screens[0] = new Screen(0, runTime.mainMonitor);
-    screens[0].initScreen();
-    int nMonitor = 0;
-    for (int i = 1; i < screens.length; i++) {
-      if (nMonitor == runTime.mainMonitor) {
+    setMouseRobot();
+    if (null == globalRobot) {
+      screens = new Screen[1];
+      screens[0] = null;
+    } else {
+      screens = new Screen[runTime.nMonitors];
+      screens[0] = new Screen(0, runTime.mainMonitor);
+      screens[0].initScreen();
+      int nMonitor = 0;
+      for (int i = 1; i < screens.length; i++) {
+        if (nMonitor == runTime.mainMonitor) {
+          nMonitor++;
+        }
+        screens[i] = new Screen(i, nMonitor);
+        screens[i].initScreen();
         nMonitor++;
       }
-      screens[i] = new Screen(i, nMonitor);
-      screens[i].initScreen();
-      nMonitor++;
-    }
-    Mouse.init();
-    Keys.init();
-    if (getNumberScreens() > 1) {
-      log(lvl, "initScreens: multi monitor mouse check");
-      Location lnow = Mouse.at();
-      float mmd = Settings.MoveMouseDelay;
-      Settings.MoveMouseDelay = 0f;
-      Location lc = null, lcn = null;
-      for (Screen s : screens) {
-        lc = s.getCenter();
-        Mouse.move(lc);
-        lcn = Mouse.at();
-        if (!lc.equals(lcn)) {
-          log(lvl, "*** multimonitor click check: %s center: (%d, %d) --- NOT OK:  (%d, %d)",
-                  s.toStringShort(), lc.x, lc.y, lcn.x, lcn.y);
-        } else {
-          log(lvl, "*** checking: %s center: (%d, %d) --- OK", s.toStringShort(), lc.x, lc.y);
+      Mouse.init();
+      if (getNumberScreens() > 1) {
+        log(lvl, "initScreens: multi monitor mouse check");
+        Location lnow = Mouse.at();
+        float mmd = Settings.MoveMouseDelay;
+        Settings.MoveMouseDelay = 0f;
+        Location lc = null, lcn = null;
+        for (Screen s : screens) {
+          lc = s.getCenter();
+          Mouse.move(lc);
+          lcn = Mouse.at();
+          if (!lc.equals(lcn)) {
+            log(lvl, "*** multimonitor click check: %s center: (%d, %d) --- NOT OK:  (%d, %d)",
+                    s.toStringShort(), lc.x, lc.y, lcn.x, lcn.y);
+          } else {
+            log(lvl, "*** checking: %s center: (%d, %d) --- OK", s.toStringShort(), lc.x, lc.y);
+          }
         }
+        Mouse.move(lnow);
+        Settings.MoveMouseDelay = mmd;
       }
-      Mouse.move(lnow);
-      Settings.MoveMouseDelay = mmd;
     }
   }
 
-  protected static IRobot getMouseRobot() {
+  public static IRobot getGlobalRobot() {
+    return globalRobot;
+  }
+
+  private static void setMouseRobot() {
     try {
-      if (globalRobot == null) {
+      if (globalRobot == null && !GraphicsEnvironment.isHeadless()) {
         globalRobot = new RobotDesktop();
       }
     } catch (AWTException e) {
       Debug.error("Can't initialize global Robot for Mouse: " + e.getMessage());
-      Sikulix.terminate(999);
+    }
+  }
+
+  private IRobot getMouseRobot() {
+    setMouseRobot();
+    if (null == globalRobot && !GraphicsEnvironment.isHeadless()) {
+      log(-1, "problem getting a java.awt.Robot");
+      Sikulix.endError(999);
     }
     return globalRobot;
   }
-  
+
   protected static Region getFakeRegion() {
     if (fakeRegion == null) {
       fakeRegion = new Region(0,0,5,5);
     }
     return fakeRegion;
   }
-  
+
   /**
    * create a Screen (ScreenUnion) object as a united region of all available monitors
    * @return ScreenUnion
    */
-//TODO: check wether this can be a Screen object, to be tested: Region methods
   public static ScreenUnion all() {
     return new ScreenUnion();
   }
@@ -150,7 +178,7 @@ public class Screen extends Region implements EventObserver, IScreen {
     curID = id;
     this.monitor = monitor;
   }
-  
+
   public static Screen as(int id) {
     if (id < 0 || id >= runTime.nMonitors) {
       Debug.error("Screen(%d) not in valid range 0 to %d - using primary %d",
@@ -175,7 +203,7 @@ public class Screen extends Region implements EventObserver, IScreen {
     } else {
 			curID = id;
 		}
-    monitor = screens[id].monitor;
+    monitor = screens[curID].monitor;
     initScreen();
   }
 
@@ -217,10 +245,7 @@ public class Screen extends Region implements EventObserver, IScreen {
     initScreen();
   }
 
-  /**
-	 * <br>TODO: remove this method if it is not needed
-	 * @param scr
-   */
+  //TODO: remove this method if it is not needed
   public void initScreen(Screen scr) {
     updateSelf();
   }
@@ -403,7 +428,15 @@ public class Screen extends Region implements EventObserver, IScreen {
    */
   @Override
   public IRobot getRobot() {
-    return robot;
+    return getMouseRobot();
+  }
+
+  protected static IRobot getRobot(Region reg) {
+    if (reg == null || null == reg.getScreen()) {
+      return getPrimaryScreen().getMouseRobot();
+    } else {
+      return reg.getScreen().getRobot();
+    }
   }
 
   /**
@@ -439,6 +472,38 @@ public class Screen extends Region implements EventObserver, IScreen {
   //</editor-fold>
 
   //<editor-fold defaultstate="collapsed" desc="Capture - SelectRegion">
+
+  public ScreenImage cmdCapture(Object... args) {
+    if (args.length == 0) {
+      return userCapture("capture an image");
+    }
+    if (args.length == 1) {
+      Object arg0 = args[0];
+      if (arg0 instanceof String) {
+        return userCapture((String) arg0);
+      } else if (arg0 instanceof Region) {
+        return capture((Region) arg0);
+      } else if (arg0 instanceof Rectangle) {
+        return capture((Rectangle) arg0);
+      }
+    }
+    if (args.length == 4) {
+      Integer argInt = null;
+      for (Object arg : args){
+        argInt = null;
+        try {
+          argInt = (Integer) arg;
+        } catch (Exception ex) {
+          break;
+        }
+      }
+      if (argInt != null) {
+        return capture((int) args[0], (int) args[1], (int) args[2], (int) args[3]);
+      }
+    }
+    return userCapture("Invalid parameter for capture");
+  }
+
   /**
    * create a ScreenImage with the physical bounds of this screen
    *
@@ -479,7 +544,7 @@ public class Screen extends Region implements EventObserver, IScreen {
     lastCaptureTime = new Date().getTime();
     ScreenImage simg = robot.captureScreen(rect);
     if (Settings.FindProfiling) {
-      Debug.logp("[FindProfiling] Screen.capture [%d x %d]: %d msec", 
+      Debug.logp("[FindProfiling] Screen.capture [%d x %d]: %d msec",
               rect.width, rect.height, new Date().getTime() - lastCaptureTime);
     }
     lastScreenImage = simg;
@@ -488,7 +553,7 @@ public class Screen extends Region implements EventObserver, IScreen {
     }
     return simg;
   }
-  
+
   /**
    * create a ScreenImage with given region on this screen
    *
@@ -500,6 +565,43 @@ public class Screen extends Region implements EventObserver, IScreen {
     return capture(reg.getRect());
   }
 
+  public static void doPrompt(String message, EventObserver obs) {
+    captureObserver = obs;
+    Screen.getPrimaryScreen().userCapture(message);
+  }
+
+  public static void closePrompt() {
+    for (int is = 0; is < Screen.getNumberScreens(); is++) {
+      if (!Screen.getScreen(is).hasPrompt()) {
+        continue;
+      }
+      Screen.getScreen(is).prompt.close();
+    }
+  }
+
+  public static void closePrompt(Screen scr) {
+    for (int is = 0; is < Screen.getNumberScreens(); is++) {
+      if (Screen.getScreen(is).getID() == scr.getID() ||
+          !Screen.getScreen(is).hasPrompt()) {
+        continue;
+      }
+      Screen.getScreen(is).prompt.close();
+      Screen.getScreen(is).prompt = null;
+    }
+  }
+
+  public static void resetPrompt(OverlayCapturePrompt ocp) {
+    int scrID = ocp.getScrID();
+    if (scrID > -1) {
+      Screen.getScreen(scrID).prompt = null;
+    }
+    resetActiveCapturePrompt();
+  }
+
+  public boolean hasPrompt() {
+    return prompt != null;
+  }
+
   /**
    * interactive capture with predefined message: lets the user capture a screen image using the
    * mouse to draw the rectangle
@@ -510,20 +612,6 @@ public class Screen extends Region implements EventObserver, IScreen {
     return userCapture("");
   }
 
-  public static void startPrompt(String message, EventObserver obs) {
-    String msg = message.isEmpty() ? promptMsg : message;
-    for (int is = 0; is < Screen.getNumberScreens(); is++) {
-      Screen.getScreen(is).prompt = new OverlayCapturePrompt(Screen.getScreen(is), obs);
-      Screen.getScreen(is).prompt.prompt(msg);
-    }
-  }
-  
-  public static void closePrompt() {
-    for (int is = 0; is < Screen.getNumberScreens(); is++) {
-      Screen.getScreen(is).prompt.close();
-    }
-  }
-  
   /**
    * interactive capture with given message: lets the user capture a screen image using the mouse to
    * draw the rectangle
@@ -533,61 +621,73 @@ public class Screen extends Region implements EventObserver, IScreen {
    */
   @Override
   public ScreenImage userCapture(final String message) {
+    if (!setActiveCapturePrompt()) {
+      return null;
+    }
+    Debug.log(3, "TRACE: Screen: userCapture");
     waitPrompt = true;
     Thread th = new Thread() {
       @Override
       public void run() {
         String msg = message.isEmpty() ? promptMsg : message;
         for (int is = 0; is < Screen.getNumberScreens(); is++) {
-          Screen.getScreen(is).prompt = new OverlayCapturePrompt(Screen.getScreen(is), Screen.this);
+          if (ignorePrimaryAtCapture && is == 0) {
+            continue;
+          }
+          Screen.getScreen(is).prompt = new OverlayCapturePrompt(Screen.getScreen(is));
+          Screen.getScreen(is).prompt.addObserver(captureObserver);
           Screen.getScreen(is).prompt.prompt(msg);
         }
       }
     };
     th.start();
-    boolean hasShot = true;
-    try {
-      int count = 0;
-      while (waitPrompt) {
-        Thread.sleep(100);
-        if (count++ > waitForScreenshot) {
-          hasShot = false;
-          break;
-        }
-      }
-    } catch (InterruptedException e) {
-      hasShot = false;
+    if (captureObserver != null) {
+      return null;
     }
+    boolean isComplete = false;
     ScreenImage simg = null;
-    if (hasShot) {
+    int count = 0;
+    while (!isComplete) {
+      this.wait(0.1f);
+      if (count++ > waitForScreenshot) {
+        break;
+      }
       for (int is = 0; is < Screen.getNumberScreens(); is++) {
-        if (simg == null) {
-          simg = Screen.getScreen(is).prompt.getSelection();
+        OverlayCapturePrompt ocp = Screen.getScreen(is).prompt;
+        if (ocp == null) {
+          continue;
+        }
+        if (ocp.isComplete()) {
+          closePrompt(Screen.getScreen(is));
+          simg = ocp.getSelection();
           if (simg != null) {
             Screen.getScreen(is).lastScreenImage = simg;
           }
+          ocp.close();
+          Screen.getScreen(is).prompt = null;
+          isComplete = true;
         }
-        Screen.getScreen(is).prompt.close();
       }
     }
+    resetActiveCapturePrompt();
     return simg;
   }
-  
+
   public String saveCapture(String name) {
     return saveCapture(name, null);
   }
-  
+
   public String saveCapture(String name, Region reg) {
-    ScreenImage img;
+    ScreenImage simg;
     if (reg == null) {
-      img = userCapture("Capture for image " + name);
+      simg = userCapture("Capture for image " + name);
     } else {
-      img = capture(reg);
+      simg = capture(reg);
     }
-    if (img == null) {
+    if (simg == null) {
       return null;
     } else {
-      return img.saveInBundle(name);
+      return simg.saveInBundle(name);
     }
   }
 
@@ -608,6 +708,7 @@ public class Screen extends Region implements EventObserver, IScreen {
    * @return the region
    */
   public Region selectRegion(final String message) {
+    Debug.log(3, "TRACE: Screen: selectRegion");
     ScreenImage sim = userCapture(message);
     if (sim == null) {
       return null;
@@ -616,16 +717,6 @@ public class Screen extends Region implements EventObserver, IScreen {
     return Region.create((int) r.getX(), (int) r.getY(),
             (int) r.getWidth(), (int) r.getHeight());
   }
-
-  /**
-   * Internal use only
-   *
-   * @param s EventSubject
-   */
-  @Override
-  public void update(EventSubject s) {
-    waitPrompt = false;
-  }
   //</editor-fold>
 
   //<editor-fold defaultstate="collapsed" desc="Visual effects">
@@ -645,8 +736,9 @@ public class Screen extends Region implements EventObserver, IScreen {
   @Override
   public String toString() {
     Rectangle r = getBounds();
-    return String.format("S(%d)[%d,%d %dx%d] E:%s, T:%.1f",
-            curID, (int) r.getX(), (int) r.getY(),
+    String scrText = curID == -1 ? "Union" : "" + curID;
+    return String.format("S(%s)[%d,%d %dx%d] E:%s, T:%.1f",
+            scrText, (int) r.getX(), (int) r.getY(),
             (int) r.getWidth(), (int) r.getHeight(),
             getThrowException() ? "Y" : "N", getAutoWaitTimeout());
   }
@@ -659,14 +751,9 @@ public class Screen extends Region implements EventObserver, IScreen {
   @Override
   public String toStringShort() {
     Rectangle r = getBounds();
-    return String.format("S(%d)[%d,%d %dx%d]",
-            curID, (int) r.getX(), (int) r.getY(),
+    String scrText = curID == -1 ? "Union" : "" + curID;
+    return String.format("S(%s)[%d,%d %dx%d]",
+            scrText, (int) r.getX(), (int) r.getY(),
             (int) r.getWidth(), (int) r.getHeight());
   }
-
-	@Override
-	public String toJSON() {
-    Rectangle r = getBounds();
-		return String.format("[\"S\", %d, %d, %d, %d, %d]", r.x, r.y, r.width, r.height, curID);
-	}
 }
