@@ -1,8 +1,7 @@
 /*
- * Copyright 2010-2014, Sikuli.org, sikulix.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * modified RaiMan
  */
 package org.sikuli.script;
 
@@ -28,4 +27,8 @@ public class SikuliException extends Exception {
         ret += "Line ?, in File ?";
         return ret;
     }
+
+    public String toStringShort() {
+        return _name + ": " + getMessage();
+    }
 }
