@@ -1,42 +1,32 @@
 /*
- * Copyright 2010-2014, Sikuli.org, SikulixUtil.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * modified RaiMan
  */
 package org.sikuli.script;
 
-import org.sikuli.basics.HotkeyManager;
-import org.sikuli.util.Tests;
+import org.sikuli.vnc.VNCScreen;
+import org.sikuli.android.ADBScreen;
+import org.sikuli.basics.*;
+import org.sikuli.util.JythonHelper;
 import org.sikuli.util.ScreenHighlighter;
-import java.awt.Dimension;
+import org.sikuli.util.SikulixFileChooser;
+import org.sikuli.util.Tests;
+
+import javax.swing.*;
+import java.awt.*;
 import java.io.File;
+import java.io.IOException;
 import java.net.URL;
 import java.security.CodeSource;
-import javax.swing.Box;
-import javax.swing.BoxLayout;
-import javax.swing.JLabel;
-import javax.swing.JOptionPane;
-import javax.swing.JPanel;
-import javax.swing.JPasswordField;
-import javax.swing.JScrollPane;
-import javax.swing.JTextArea;
-import org.sikuli.basics.Debug;
-import org.sikuli.basics.FileManager;
-import org.sikuli.basics.PreferencesUser;
-import org.sikuli.basics.Settings;
-import org.sikuli.util.JythonHelper;
 
 /**
- * global services for package API
+ * INTERNAL USE ONLY --- NOT part of official API
  */
 public class Sikulix {
 
-  private static String imgLink = "http://www.sikulix.com/uploads/1/4/2/8/14281286";
-  private static String imgHttp = "1389888228.jpg";
-  private static String imgNet = imgLink + "/" + imgHttp;
-
   private static int lvl = 3;
+
   private static void log(int level, String message, Object... args) {
     Debug.logx(level, "Sikulix: " + message, args);
   }
@@ -57,11 +47,12 @@ public class Sikulix {
   private static RunTime rt = null;
   public static int testNumber = -1;
   private static boolean shouldRunServer = false;
+  private static Point locPopAt = null;
 
   static {
     String jarName = "";
 
-    CodeSource codeSrc =  Sikulix.class.getProtectionDomain().getCodeSource();
+    CodeSource codeSrc = Sikulix.class.getProtectionDomain().getCodeSource();
     if (codeSrc != null && codeSrc.getLocation() != null) {
       jarName = codeSrc.getLocation().getPath();
     }
@@ -102,8 +93,6 @@ public class Sikulix {
     if (args.length > 0 && args[0].toLowerCase().startsWith("-s")) {
       shouldRunServer = true;
     } else {
-      System.out.println("********** Running Sikulix.main");
-
       int dl = RunTime.checkArgs(args, RunTime.Type.API);
       if (dl > -1 && dl < 999) {
         testNumber = dl;
@@ -132,44 +121,23 @@ public class Sikulix {
       Sikulix.popError("Not useable!\nRun setup first!");
       System.exit(0);
     }
-    
+
     if (shouldRunServer) {
       if (RunServer.run(null)) {
         System.exit(1);
       }
     }
-    
-    Debug.on(3);
-    //******Test Space*************************************
-//    Screen.showMonitors();
-//    Screen s = Screen.as(0);
 
-//    System.exit(1);
-    
-//    Run.connect();
-//    p(Run.show());
-//    p(Run.send("START"));
-//    p(Run.send("EVAL?r=Region.create(100,100,100,100);r.toJSON();"));
-//    Run.close();
-//    Run.stop();
-    //*********************************
-
-    Settings.InfoLogs = false;
-    Settings.ActionLogs = false;
-    
-    ImagePath.add("org.sikuli.script.Sikulix/ImagesAPI.sikuli");
-
-    if (rt.runningWinApp) {
-      popup("Hello World\nNot much else to do ( yet ;-)", rt.fSxBaseJar.getName());
-      try {
-        Screen scr = new Screen();
-        scr.find(new Image(scr.userCapture("grab something to find"))).highlight(3);
-      } catch (Exception ex) {
-        popup("Uuups :-(\n" + ex.getMessage(), rt.fSxBaseJar.getName());
-      }
-      popup("Hello World\nNothing else to do ( yet ;-)", rt.fSxBaseJar.getName());
+    boolean playing = false;
+    if (args.length > 0 && "play".equals(args[0])) {
+      playing = true;
+//-------- playground
+// -------- playground
+    }
+    if (playing) {
       System.exit(1);
     }
+
     String version = String.format("(%s-%s)", rt.getVersionShort(), rt.sxBuildStamp);
     File lastSession = new File(rt.fSikulixStore, "LastAPIJavaScript.js");
     String runSomeJS = "";
@@ -177,31 +145,32 @@ public class Sikulix {
       runSomeJS = FileManager.readFileToString(lastSession);
     }
     runSomeJS = inputText("enter some JavaScript (know what you do - may silently die ;-)"
-            + "\nexample: run(\"git*\") will run the JavaScript showcase from GitHub"
-            + "\nWhat you enter now will be shown the next time.",
+                    + "\nexample: run(\"git*\") will run the JavaScript showcase from GitHub"
+                    + "\nWhat you enter now will be shown the next time.",
             "API::JavaScriptRunner " + version, 10, 60, runSomeJS);
-    if (runSomeJS.isEmpty()) {
+    if (runSomeJS == null || runSomeJS.isEmpty()) {
       popup("Nothing to do!", version);
     } else {
       while (!runSomeJS.isEmpty()) {
         FileManager.writeStringToFile(runSomeJS, lastSession);
         Runner.runjs(null, null, runSomeJS, null);
         runSomeJS = inputText("Edit the JavaScript and/or press OK to run it (again)\n"
-                + "Press Cancel to terminate",
-            "API::JavaScriptRunner " + version, 10, 60, runSomeJS);
+                        + "Press Cancel to terminate",
+                "API::JavaScriptRunner " + version, 10, 60, runSomeJS);
       }
     }
     System.exit(0);
   }
-  
+
   /**
    * add a jar to the scripting environment<br>
    * Jython: added to sys.path<br>
    * JRuby: not yet supported<br>
    * JavaScript: not yet supported<br>
    * if no scripting active (API usage), jar is added to classpath if available
-   * @param fpJar absolute path to a jar (relative: searched according to Extension concept, 
-   * but first on sys.path)
+   *
+   * @param fpJar absolute path to a jar (relative: searched according to Extension concept,
+   *              but first on sys.path)
    * @return the absolute path to the jar or null, if not available
    */
   public static String load(String fpJar) {
@@ -215,8 +184,9 @@ public class Sikulix {
    * JavaScript: only added to classpath<br>
    * if no scripting is active (API usage), jar is added to classpath if available<br>
    * additionally: fpJar/fpJarImagePath is added to ImagePath (not checked)
-   * @param fpJar absolute path to a jar (relative: searched according to Extension concept, 
-   * but first on sys.path)
+   *
+   * @param fpJar          absolute path to a jar (relative: searched according to Extension concept,
+   *                       but first on sys.path)
    * @param fpJarImagePath path relative to jar root inside jar
    * @return the absolute path to the jar or null, if not available
    */
@@ -234,21 +204,22 @@ public class Sikulix {
       if (fJarFound != null) {
         fpJarFound = fJarFound.getAbsolutePath();
         rt.addToClasspath(fpJarFound);
-      } 
+      }
     }
     if (fpJarFound != null && fpJarImagePath != null) {
       ImagePath.addJar(fpJarFound, fpJarImagePath);
     }
     return fpJarFound;
   }
-  
+
   /**
    * build a jar on the fly at runtime from a folder.<br>
-   * special for Jython: if the folder contains a __init__.py on first level, 
+   * special for Jython: if the folder contains a __init__.py on first level,
    * the folder will be copied to the jar root (hence preserving module folders)
-   * @param targetJar absolute path to the created jar (parent folder must exist, jar is overwritten) 
-   * @param sourceFolder absolute path to a folder, the contained folder structure 
-   * will be copied to the jar root level
+   *
+   * @param targetJar    absolute path to the created jar (parent folder must exist, jar is overwritten)
+   * @param sourceFolder absolute path to a folder, the contained folder structure
+   *                     will be copied to the jar root level
    * @return
    */
   public static boolean buildJarFromFolder(String targetJar, String sourceFolder) {
@@ -270,19 +241,20 @@ public class Sikulix {
         prefix = prefix.substring(0, prefix.length() - 1);
       }
     }
-    return FileManager.buildJar(targetJar, new String[]{null}, 
-            new String[] {sourceFolder}, new String[] {prefix}, null);
+    return FileManager.buildJar(targetJar, new String[]{null},
+            new String[]{sourceFolder}, new String[]{prefix}, null);
   }
-  
+
   /**
-   * the foo.py files in the given source folder are compiled to JVM-ByteCode-classfiles foo$py.class 
+   * the foo.py files in the given source folder are compiled to JVM-ByteCode-classfiles foo$py.class
    * and stored in the target folder (thus securing your code against changes).<br>
    * A folder structure is preserved. All files not ending as .py will be copied also.
    * The target folder might then be packed to a jar using buildJarFromFolder.<br>
-   * Be aware: you will get no feedback about any compile problems, 
+   * Be aware: you will get no feedback about any compile problems,
    * so make sure your code compiles error free. Currently there is no support for running such a jar,
    * it can only be used with load()/import, but you might provide a simple script that does load()/import
    * and then runs something based on available functions in the jar code.
+   *
    * @param fpSource absolute path to a folder/folder-tree containing the stuff to be copied/compiled
    * @param fpTarget the folder that will contain the copied/compiled stuff (folder is first deleted)
    * @return false if anything goes wrong, true means should have worked
@@ -315,11 +287,11 @@ public class Sikulix {
     }
     return false;
   }
-  
+
   private static class CompileJythonFilter implements FileManager.FileFilter {
-    
+
     JythonHelper jython = null;
-    
+
     public CompileJythonFilter(JythonHelper jython) {
       this.jython = jython;
     }
@@ -332,7 +304,7 @@ public class Sikulix {
       return false;
     }
   }
-  
+
   private static JythonHelper doCompileJythonFolder(JythonHelper jython, File fSource) {
     String fpSource = FileManager.slashify(fSource.getAbsolutePath(), false);
     if (!jython.exec(String.format("compileall.compile_dir(\"%s\","
@@ -401,7 +373,6 @@ public class Sikulix {
     }
 //TODO collect initializations here
     Mouse.init();
-    Keys.init();
     return new Screen();
   }
 
@@ -462,31 +433,43 @@ public class Sikulix {
         popError(String.format(msg, n));
       }
     } else {
-      Debug.error("***** Terminating SikuliX after a fatal error"
+      msg = "***** Terminating SikuliX after a fatal error"
               + (n == 0 ? "*****\n" : " %d *****\n")
               + "It makes no sense to continue!\n"
               + "If you do not have any idea about the error cause or solution, run again\n"
-              + "with a Debug level of 3. You might paste the output to the Q&A board.", n);
-      cleanUp(0);
+              + "with a Debug level of 3. You might paste the output to the Q&A board.";
+      log(-1, msg, n);
+      if (Settings.isRunningIDE) {
+        popError(String.format(msg, n));
+      }
+      cleanUp(n);
+    }
+    if (!Settings.isRunningIDE) {
+      System.exit(1);
+    } else {
+      throw new IllegalStateException("Aborting script due to an internal error - see log");
     }
-    System.exit(1);
   }
 
   /**
-   * INTERNAL USE: resets stateful Sikuli X features: ScreenHighlighter, Observing, Mouse, Key, Hotkeys When in IDE:
-   * resets selected options to defaults (TODO)
+   * INTERNAL USE: resets stateful Sikuli X features: <br>
+   * ScreenHighlighter, Observing, Mouse, Key, Hotkeys <br>
+   * When in IDE: resets selected options to defaults (TODO)
    *
    * @param n returncode
    */
   public static void cleanUp(int n) {
     log(lvl, "cleanUp: %d", n);
+    VNCScreen.stopAll();
+    ADBScreen.stop();
     ScreenHighlighter.closeAll();
     Observing.cleanUp();
-    Mouse.reset();
-    //TODO move to class Keys after implementation
-    Screen.getPrimaryScreen().getRobot().keyUp();
-    //TODO what about remote screen sessions????
     HotkeyManager.reset();
+    try {
+      new RobotDesktop().keyUp();
+    } catch (AWTException e) {
+    }
+    Mouse.reset();
   }
 
   /**
@@ -577,7 +560,11 @@ public class Sikulix {
   }
 
   public static void popError(String message, String title) {
-    JOptionPane.showMessageDialog(null, message, title, JOptionPane.ERROR_MESSAGE);
+    JFrame anchor = popLocation();
+    JOptionPane.showMessageDialog(anchor, message, title, JOptionPane.ERROR_MESSAGE);
+    if (anchor != null) {
+      anchor.dispose();
+    }
   }
 
   /**
@@ -593,11 +580,14 @@ public class Sikulix {
    * @return the text entered
    */
   public static String input(String msg, String preset, String title, boolean hidden) {
+    JFrame anchor = popLocation();
+    String ret = "";
     if (!hidden) {
       if ("".equals(title)) {
         title = "Sikuli input request";
       }
-      return (String) JOptionPane.showInputDialog(null, msg, title, JOptionPane.PLAIN_MESSAGE, null, null, preset);
+      ret = (String) JOptionPane.showInputDialog(anchor, msg, title,
+              JOptionPane.PLAIN_MESSAGE, null, null, preset);
     } else {
       preset = "";
       JTextArea tm = new JTextArea(msg);
@@ -612,18 +602,19 @@ public class Sikulix {
       pnl.add(pw);
       pnl.add(Box.createVerticalStrut(10));
       pnl.add(tm);
-      if (0 == JOptionPane.showConfirmDialog(null, pnl, title, JOptionPane.OK_CANCEL_OPTION)) {
+      int retval = JOptionPane.showConfirmDialog(anchor, pnl, title, JOptionPane.OK_CANCEL_OPTION);
+      if (0 == retval) {
         char[] pwc = pw.getPassword();
-        String pwr = "";
         for (int i = 0; i < pwc.length; i++) {
-          pwr = pwr + pwc[i];
+          ret = ret + pwc[i];
           pwc[i] = 0;
         }
-        return pwr;
-      } else {
-        return "";
       }
     }
+    if (anchor != null) {
+      anchor.dispose();
+    }
+    return ret;
   }
 
   public static String input(String msg, String title, boolean hidden) {
@@ -654,7 +645,11 @@ public class Sikulix {
     if (title == null) {
       title = "... something to decide!";
     }
-    int ret = JOptionPane.showConfirmDialog(null, msg, title, JOptionPane.YES_NO_OPTION);
+    JFrame anchor = popLocation();
+    int ret = JOptionPane.showConfirmDialog(anchor, msg, title, JOptionPane.YES_NO_OPTION);
+    if (anchor != null) {
+      anchor.dispose();
+    }
     if (ret == JOptionPane.CLOSED_OPTION || ret == JOptionPane.NO_OPTION) {
       return false;
     }
@@ -666,7 +661,59 @@ public class Sikulix {
   }
 
   public static void popup(String message, String title) {
-    JOptionPane.showMessageDialog(null, message, title, JOptionPane.PLAIN_MESSAGE);
+    JFrame anchor = popLocation();
+    JOptionPane.showMessageDialog(anchor, message, title, JOptionPane.PLAIN_MESSAGE);
+    if (anchor != null) {
+      anchor.dispose();
+    }
+  }
+
+  public static Location popat(Location at) {
+    locPopAt = new Point(at.x, at.y);
+    return new Location(locPopAt);
+  }
+
+  public static Location popat(Region at) {
+    locPopAt = new Point(at.getCenter().x, at.getCenter().y);
+    return new Location(locPopAt);
+  }
+
+  public static Location popat(int atx, int aty) {
+    locPopAt = new Point(atx, aty);
+    return new Location(locPopAt);
+  }
+
+  public static Location popat() {
+    locPopAt = getLocPopAt();
+    return new Location(locPopAt);
+  }
+
+  private static Point getLocPopAt() {
+    Rectangle screen0 = rt.getMonitor(0);
+    if (null == screen0) {
+      return null;
+    }
+    return new Point((int) screen0.getCenterX(), (int) screen0.getCenterY());
+  }
+
+  private static JFrame popLocation() {
+    if (null == locPopAt) {
+      locPopAt = getLocPopAt();
+      if (null == locPopAt) {
+        return null;
+      }
+    }
+    return popLocation(locPopAt.x, locPopAt.y);
+  }
+
+  private static JFrame popLocation(int x, int y) {
+    JFrame anchor = new JFrame();
+    anchor.setAlwaysOnTop(true);
+    anchor.setUndecorated(true);
+    anchor.setSize(1, 1);
+    anchor.setLocation(x, y);
+    anchor.setVisible(true);
+    return anchor;
   }
 
   public static String popSelect(String msg, String[] options, String preset) {
@@ -697,58 +744,108 @@ public class Sikulix {
     if (preset == null) {
       preset = options[0];
     }
-    return (String) JOptionPane.showInputDialog(null, msg, title, JOptionPane.PLAIN_MESSAGE, null, options, preset);
+    JFrame anchor = popLocation();
+    String ret = (String) JOptionPane.showInputDialog(anchor, msg, title,
+            JOptionPane.PLAIN_MESSAGE, null, options, preset);
+    if (anchor != null) {
+      anchor.dispose();
+    }
+    return ret;
+  }
+
+  public static String popFile(String title) {
+    popat(new Screen(0).getCenter());
+    JFrame anchor = popLocation();
+    SikulixFileChooser fc = new SikulixFileChooser(anchor);
+    File ret = fc.show(title);
+    fc = null;
+    if (anchor != null) {
+      anchor.dispose();
+    }
+    if (ret == null) {
+      return "";
+    }
+    return ret.getAbsolutePath();
+  }
+
+  public static String inputText(String msg) {
+    return inputText(msg, "", 0, 0, "");
+  }
+
+  public static String inputText(String msg, int lines, int width) {
+    return inputText(msg, "", lines, width, "");
+  }
+
+  public static String inputText(String msg, int lines, int width, String text) {
+    return inputText(msg, "", lines, width, text);
+  }
+
+  public static String inputText(String msg, String text) {
+    return inputText(msg, "", 0, 0, text);
   }
 
   /**
    * Shows a dialog request to enter text in a multiline text field <br>
-   * Though not all text might be visible, everything entered is delivered with the returned text <br>
-   * The main purpose for this feature is to allow pasting text from somewhere preserving line breaks <br>
+   * it has line wrapping on word bounds and a vertical scrollbar if needed
    *
-   * @param msg the message to display.
-   * @param title the title for the dialog (default: Sikuli input request)
+   * @param msg   the message to display below the textfield
+   * @param title the title for the dialog (default: SikuliX input request)
    * @param lines the maximum number of lines visible in the text field (default 9)
-   * @param width the maximum number of characters visible in one line (default 20)
+   * @param width the maximum number of characters visible in one line (default 20 letters m)
+   * @param text  a preset text to show
    * @return The user's input including the line breaks.
    */
-  public static String inputText(String msg, String title, int lines, int width) {
-    return inputText(msg, title, lines, width, "");
-  }
-
   public static String inputText(String msg, String title, int lines, int width, String text) {
     width = Math.max(20, width);
     lines = Math.max(9, lines);
     if ("".equals(title)) {
-      title = "Sikuli input request";
+      title = "SikuliX input request";
     }
     JTextArea ta = new JTextArea("");
-    int w = width * ta.getFontMetrics(ta.getFont()).charWidth('m');
-    int h = (int) (lines * ta.getFontMetrics(ta.getFont()).getHeight());
-    ta.setPreferredSize(new Dimension(w, h));
-    ta.setMaximumSize(new Dimension(w, 2 * h));
+    String fontname = "Dialog";
+    int pluswidth = 1;
+    if (Settings.InputFontMono) {
+      fontname = "Monospaced";
+      pluswidth = 3;
+    }
+    ta.setFont(new Font(fontname, Font.PLAIN, Math.max(14, Settings.InputFontSize)));
+    int w = (width + pluswidth) * ta.getFontMetrics(ta.getFont()).charWidth('m');
+    int h = (lines + 1) * ta.getFontMetrics(ta.getFont()).getHeight();
     ta.setText(text);
-    JScrollPane sp = new JScrollPane(ta);
-    sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
-    JTextArea tm = new JTextArea(msg);
+    ta.setLineWrap(true);
+    ta.setWrapStyleWord(true);
+
+    JScrollPane sp = new JScrollPane();
+    sp.setViewportView(ta);
+    sp.setPreferredSize(new Dimension(w, h));
+
+    JTextArea tm = new JTextArea("");
+    tm.setFont(new Font(fontname, Font.PLAIN, Math.max(14, Settings.InputFontSize)));
     tm.setColumns(width);
+    tm.setText(msg);
     tm.setLineWrap(true);
     tm.setWrapStyleWord(true);
     tm.setEditable(false);
     tm.setBackground(new JLabel().getBackground());
+
     JPanel pnl = new JPanel();
     pnl.setLayout(new BoxLayout(pnl, BoxLayout.Y_AXIS));
     pnl.add(sp);
     pnl.add(Box.createVerticalStrut(10));
     pnl.add(tm);
     pnl.add(Box.createVerticalStrut(10));
-    if (0 == JOptionPane.showConfirmDialog(null, pnl, title, JOptionPane.OK_CANCEL_OPTION)) {
+    JFrame anchor = popLocation();
+    int ret = JOptionPane.showConfirmDialog(anchor, pnl, title, JOptionPane.OK_CANCEL_OPTION);
+    if (anchor != null) {
+      anchor.dispose();
+    }
+    if (0 == ret) {
       return ta.getText();
     } else {
-      return "";
+      return null;
     }
   }
 
-
   public static boolean importPrefs(String path) {
     return true;
   }
@@ -773,7 +870,7 @@ public class Sikulix {
    * information between IDE sessions<br>
    * this allows, to easily make some valuable information persistent
    *
-   * @param key name of the item
+   * @param key   name of the item
    * @param value item content
    */
   public static void prefStore(String key, String value) {
@@ -795,7 +892,7 @@ public class Sikulix {
    * retrieve the value of a previously stored a key-value-pair from Javas persistent preferences storage that is used
    * by SikuliX to save settings and information between IDE sessions<br>
    *
-   * @param key name of the item
+   * @param key   name of the item
    * @param value the item content or the given value if not stored yet (default)
    * @return the item content or the given default
    */
@@ -824,7 +921,41 @@ public class Sikulix {
     PreferencesUser.getInstance().removeAll(prefNonSikuli);
   }
 
-  private static Exception Exception() {
-    throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
+  /**
+   * convenience for a password protected VNCScreen connection
+   * (use theVNCScreen.stop() to stop the connection)
+   * active screens are auto-stopped at cleanup
+   *
+   * @param theIP    the server IP
+   * @param thePort  the port number
+   * @param password a needed password for the server in plain text
+   * @param cTimeout seconds to wait for a valid connection
+   * @param timeout  value in milli-seconds during normal operation
+   * @return a VNCScreen object
+   */
+  public static VNCScreen vncStart(String theIP, int thePort, String password, int cTimeout, int timeout) {
+    try {
+      return VNCScreen.start(theIP, thePort, password, cTimeout, timeout);
+    } catch (IOException e) {
+      throw new RuntimeException(e);
+    }
+  }
+
+  /**
+   * convenience for a VNCScreen connection (use theVNCScreen.stop() to stop the connection)
+   * active screens are auto-stopped at cleanup
+   *
+   * @param theIP    the server IP
+   * @param thePort  the port number
+   * @param cTimeout seconds to wait for a valid connection
+   * @param timeout  value in milli-seconds during normal operation
+   * @return a VNCScreen object
+   */
+  public static VNCScreen vncStart(String theIP, int thePort, int cTimeout, int timeout) {
+    try {
+      return VNCScreen.start(theIP, thePort, cTimeout, timeout);
+    } catch (IOException e) {
+      throw new RuntimeException(e);
+    }
   }
 }
