@@ -1,5 +1,5 @@
 /*
- * Copyright 2010-2014, Sikuli.org, sikulix.com
+ * Copyright 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
  * WhoIsWho 2014
@@ -7,42 +7,34 @@
 package org.sikuli.scriptrunner;
 
 //import java.io.File;
-import org.sikuli.basics.Debug;
-
-import java.io.File;
 import java.io.BufferedReader;
+import java.io.File;
 import java.io.FileInputStream;
 import java.io.InputStreamReader;
-//import java.io.InputStream;
 import java.io.PipedInputStream;
 import java.io.PipedOutputStream;
 import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.List;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-import java.util.List;
-//import java.io.PrintWriter;
-//import java.io.StringWriter;
-
-import org.sikuli.basics.FileManager;
-
-import org.jruby.embed.ScriptingContainer;
-import org.jruby.javasupport.JavaEmbedUtils.EvalUnit;
-import org.jruby.CompatVersion;
-import org.jruby.Ruby;
-import org.jruby.embed.LocalContextScope;
+import org.jruby.CompatVersion;import org.jruby.Ruby;
 import org.jruby.RubyInstanceConfig.CompileMode;
 import org.jruby.RubyProc;
+import org.jruby.embed.LocalContextScope;
+import org.jruby.embed.ScriptingContainer;
+import org.jruby.javasupport.JavaEmbedUtils.EvalUnit;
 import org.jruby.javasupport.JavaUtil;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.sikuli.basics.Debug;
+import org.sikuli.basics.FileManager;
 import org.sikuli.script.RunTime;
 import org.sikuli.script.Runner;
-import org.sikuli.script.Sikulix;
 
 public class JRubyScriptRunner implements IScriptRunner {
-  
+
   static RunTime sxRunTime = RunTime.get();
 
 	//<editor-fold defaultstate="collapsed" desc="new logging concept">
@@ -92,7 +84,7 @@ public class JRubyScriptRunner implements IScriptRunner {
 
 	private boolean isFromIDE;
 	private boolean isCompileOnly;
-  
+
   private static Ruby runtime;
   private static ThreadContext context;
 
@@ -253,7 +245,7 @@ public class JRubyScriptRunner implements IScriptRunner {
 
   private boolean runObserveCallback(Object[] args) {
     if (runtime == null) {
-      runtime = Ruby.newInstance();
+      runtime = ((RubyProc) args[0]).getRuntime();
       context = runtime.getCurrentContext();
     }
     IRubyObject[] pargs;
@@ -261,7 +253,7 @@ public class JRubyScriptRunner implements IScriptRunner {
       pargs = new IRubyObject[] {JavaUtil.convertJavaToRuby(runtime, args[1])};
       ((RubyProc) args[0]).call(context, pargs);
     } catch (Exception ex) {
-      log(-1, "runObserveCallback: jruby invoke: %s\n%s\n%s", args[0].getClass(),args[0], ex.getMessage());    
+      log(-1, "runObserveCallback: jruby invoke: %s\n%s\n%s", args[0].getClass(),args[0], ex.getMessage());
     }
     return true;
   }
@@ -518,7 +510,7 @@ public class JRubyScriptRunner implements IScriptRunner {
 			return;
 		}
 		List<String> path = interpreter.getLoadPaths();
-		if (!FileManager.pathEquals((String) path.get(0), sikuliLibPath)) {
+		if (path.size() == 0 || !FileManager.pathEquals((String) path.get(0), sikuliLibPath)) {
 			log(lvl, "executeScriptHeader: adding SikuliX Lib path to sys.path\n" + sikuliLibPath);
 			int pathLength = path.size();
 			String[] pathNew = new String[pathLength + 1];
