@@ -1,8 +1,7 @@
 /*
- * Copyright 2010-2014, Sikuli.org, sikulix.com
+ * Copyright (c) 2010-2016, Sikuli.org, sikulix.com
  * Released under the MIT License.
  *
- * RaiMan 2013
  */
 package org.sikuli.setup;
 
@@ -15,6 +14,8 @@ import java.io.File;
 import java.io.FilenameFilter;
 import java.io.IOException;
 import java.lang.reflect.Method;
+import java.net.MalformedURLException;
+import java.net.URL;
 import java.net.URLClassLoader;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -25,9 +26,10 @@ import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import java.util.zip.ZipEntry;
-import javax.swing.JFrame;
+import javax.swing.*;
 import javax.swing.border.Border;
 import javax.swing.border.LineBorder;
+
 import org.rauschig.jarchivelib.Archiver;
 import org.rauschig.jarchivelib.ArchiverFactory;
 import org.sikuli.basics.Debug;
@@ -36,7 +38,7 @@ import org.sikuli.basics.SplashFrame;
 import org.sikuli.basics.PreferencesUser;
 import org.sikuli.script.RunTime;
 import org.sikuli.basics.Settings;
-import org.sikuli.script.Sikulix;
+//import org.sikuli.script.Sikulix;
 import org.sikuli.util.LinuxSupport;
 
 public class RunSetup {
@@ -49,7 +51,7 @@ public class RunSetup {
   private static File fWorkDir;
   private static String logfile;
   private static String version;
-//TODO wrong if version number parts have more than one digit
+  //TODO wrong if version number parts have more than one digit
   private static String minorversion;
   private static String majorversion;
   private static String updateVersion;
@@ -102,7 +104,7 @@ public class RunSetup {
   private static String linuxDistro = "*** testing Linux ***";
   private static String osarch;
 
-//TODO set true to test on Mac
+  //TODO set true to test on Mac
   private static boolean isLinux = false;
 
   private static boolean libsProvided = false;
@@ -116,32 +118,33 @@ public class RunSetup {
   private static boolean clean = false;
   private static RunTime runTime;
   private static File fDownloadsGenericApp;
-	private static boolean useLibsProvided = false;
+  private static boolean useLibsProvided = false;
   private static File fDownloadsObsolete;
   private static boolean runningWithProject = false;
   private static boolean shouldBuildVision = false;
-	private static boolean bequiet = false;
+  private static boolean bequiet = false;
   private static String sikulixMavenGroup = "com/sikulix/";
-	private static boolean testingMaven = false;
   private static boolean withExtensions = false;
 
-  static Map <String, String> downloadsLookfor = new HashMap<String, String>();
-  static Map <String, File> downloadsFound = new HashMap<String, File>();
-  
+  static Map<String, String> downloadsLookfor = new HashMap<String, String>();
+  static Map<String, File> downloadsFound = new HashMap<String, File>();
+
   private static File fSetupStuff = null;
   private static boolean hasAPI = true;
   private static String[] jarsList = new String[]{
-      null, // ide
-      null, // api
-      null, // tess
-      null, // jython
-      null, // jruby
-      null, // jruby+
-      null, // libwin
-      null, // libmac
-      null // liblux
-    };
+          null, // ide
+          null, // api
+          null, // tess
+          null, // jython
+          null, // jruby
+          null, // jruby+
+          null, // libwin
+          null, // libmac
+          null // liblux
+  };
 
+  private static boolean testingMaven = false;
+  private static boolean useLocalMavenRepo = false;
 
   //<editor-fold defaultstate="collapsed" desc="new logging concept">
   private static void logp(String message, Object... args) {
@@ -164,18 +167,12 @@ public class RunSetup {
 
     runTime = RunTime.get(RunTime.Type.SETUP, args);
 
-//    logp("**** command line args: %d", args.length);
-//    if (args.length > 0) {
-//      int i = 0;
-//      for (String arg : args) {
-//        logp("%3d: %s", i++, arg);
-//      }
-//    }
     version = runTime.getVersionShort();
-//TODO wrong if version number parts have more than one digit
     minorversion = runTime.getVersionShort().substring(0, 5);
     majorversion = runTime.getVersionShort().substring(0, 3);
 
+    if (args.length > 0 && "test".equals(args[0])) testingMaven = true;
+
     localSetup = String.format("sikulixsetup-%s-%s-project.jar", version, runTime.sxBuildStamp);
     if (runTime.fSxBaseJar.getPath().contains(localSetup)) {
       runningWithProject = true;
@@ -224,7 +221,7 @@ public class RunSetup {
     }
 
     options.addAll(Arrays.asList(args));
-    
+
     //<editor-fold defaultstate="collapsed" desc="options return version">
     if (args.length > 0 && "stamp".equals(args[0])) {
       System.out.println(runTime.SikuliProjectVersion + "-" + runTime.sxBuildStamp);
@@ -232,12 +229,12 @@ public class RunSetup {
     }
 
     if (args.length > 0 && "frommavenforsed".equals(args[0])) {
-			bequiet = true;
-			String name = getMavenJarPath(args[2]);
-			if (name == null) {
-				name = runTime.dlMavenSnapshot + sikulixMavenGroup;
-			}
-			name = name.replaceAll("/", "\\\\/");
+      bequiet = true;
+      String name = getMavenJarPath(args[2]);
+      if (name == null) {
+        name = runTime.dlMavenSnapshot + sikulixMavenGroup;
+      }
+      name = name.replaceAll("/", "\\\\/");
       System.out.println(name);
       System.exit(0);
     }
@@ -283,8 +280,16 @@ public class RunSetup {
       options.remove(0);
     }
 
-//TODO add parameter for proxy settings, linux options
-    if (args.length > 0 && "options".equals(args[0])) {
+    if (options.size() > 0 && "test".equals(options.get(0))) {
+      testingMaven = true;
+      if (runTime.isVersionRelease()) {
+        useLocalMavenRepo = true;
+      }
+      options.remove(0);
+    }
+
+    //TODO add parameter for proxy settings, linux options
+    if (options.size() > 0 && "options".equals(options.get(0))) {
       options.remove(0);
       if (!options.isEmpty()) {
         for (String val : options) {
@@ -333,14 +338,24 @@ public class RunSetup {
           } else if (val.toLowerCase().startsWith("clean")) {
             clean = true;
           } else if (val.toLowerCase().startsWith("ext")) {
-           hasOptions = true;
-           withExtensions = true;
+            hasOptions = true;
+            withExtensions = true;
           }
         }
         options.clear();
       }
     }
 
+    if (!hasOptions && !testingMaven) {
+      String msg = String.format("You are about to run a setup for %s (%s)", version, runTime.sxBuildStamp);
+      msg += "\n\nYou should have a suitable backup, ";
+      msg += "\nto go back in case to what you have now.";
+      msg += "\n\nClick NO to stop here";
+      if (!popAsk(msg)) {
+        userTerminated("");
+      }
+    }
+
     localLogfile = "SikuliX-" + version + "-SetupLog.txt";
 
     if (options.size() > 0) {
@@ -353,12 +368,12 @@ public class RunSetup {
       }
       log(lvl, "Seems to be run using mvn exec:exec");
     } else {
-      logPlus(lvl, "command line options:");
+      log(lvl, "command line options:");
       String strArgs = "";
       for (String arg : args) {
         strArgs += arg + " ";
       }
-      logPlus(lvl, "%s", strArgs);
+      log(lvl, "%s", strArgs);
     }
 
     //<editor-fold defaultstate="collapsed" desc="general preps">
@@ -378,14 +393,13 @@ public class RunSetup {
     }
     fDownloadsObsolete = new File(fWorkDir, "Downloads");
     workDir = fWorkDir.getAbsolutePath();
-    
+
     fSetupStuff = new File(fWorkDir, "SetupStuff");
     FileManager.resetFolder(fSetupStuff);
 
     osarch = "" + runTime.javaArch;
     if (runTime.runningLinux) {
       linuxDistro = runTime.linuxDistro;
-      logPlus(lvl, "LinuxDistro: %s (%s-Bit)", linuxDistro, osarch);
       isLinux = true;
     }
 
@@ -407,8 +421,11 @@ public class RunSetup {
     }
 
     checkDownloads();
-    
-    logToFile = true;
+
+//    if (!testingMaven) {
+//      logToFile = true;
+//    }
+
     if (logToFile) {
       logfile = (new File(fWorkDir, localLogfile)).getAbsolutePath();
       if (!Debug.setLogFile(logfile)) {
@@ -419,18 +436,22 @@ public class RunSetup {
     }
 
     if (args.length > 0) {
-      logPlus(lvl, "... starting with: " + Sikulix.arrayToString(args));
+      logPlus(lvl, "... starting with: " + arrayToString(args));
     } else {
       logPlus(lvl, "... starting with no args given");
     }
 
+    if (isLinux) {
+      logPlus(lvl, "LinuxDistro: %s (%s-Bit)", linuxDistro, osarch);
+    }
+
     logPlus(lvl, "Setup: %s %s in folder:\n%s", runTime.getVersionShort(), runTime.SikuliVersionBuild, fWorkDir);
 
     File localJarIDE = new File(fWorkDir, localIDE);
     File localJarAPI = new File(fWorkDir, localAPI);
 
     folderLibsWin = new File(fSetupStuff, "sikulixlibs/windows");
-    folderLib =  new File(fSetupStuff, "Lib");
+    folderLib = new File(fSetupStuff, "Lib");
     folderLibsLux = runTime.fLibsProvided;
 
     //TODO Windows 8 HKLM/SOFTWARE/JavaSoft add Prefs ????
@@ -439,8 +460,7 @@ public class RunSetup {
       if (popAsk(String.format("Found a libs folder at\n%s\n"
               + "Click YES to use the contained libs "
               + "for setup (be sure they are useable).\n"
-              + "Click NO to make a clean setup (libs are deleted).", folderLibsLux)))
-      {
+              + "Click NO to make a clean setup (libs are deleted).", folderLibsLux))) {
         useLibsProvided = true;
       } else {
         FileManager.resetFolder(folderLibsLux);
@@ -531,7 +551,7 @@ public class RunSetup {
       Settings.proxyChecked = true;
       //</editor-fold>
 
-    //<editor-fold defaultstate="collapsed" desc="evaluate setup options">
+      //<editor-fold defaultstate="collapsed" desc="evaluate setup options">
       if (winSU.option1.isSelected()) {
         getIDE = true;
         if (winSU.option2.isSelected()) {
@@ -662,7 +682,7 @@ public class RunSetup {
     String targetJar;
     boolean downloadOK = true;
     boolean dlOK = true;
-		downloadOK = true;
+    downloadOK = true;
 //    String dlDirBuild = fDownloadsBuild.getAbsolutePath();
     String dlDirGenericApp = fDownloadsGenericApp.getAbsolutePath();
     String dlDirGeneric = fDownloadsGeneric.getAbsolutePath();
@@ -677,11 +697,22 @@ public class RunSetup {
         forSystemMac = Settings.isMac();
       }
     }
-    File fDownloaded;
+    File fDownloaded = null;
     String sDownloaded;
     String sDownloadedName;
 
     //<editor-fold defaultstate="collapsed" desc="download lib jars">
+    if (testingMaven) {
+      fDownloaded = downloadJarFromMavenSx("sikulixsetup#forsetup", fWorkDir.getAbsolutePath(), "sikulixsetup");
+      if (fDownloaded == null) {
+        logPlus(-1, "%s not possible to copy from local MavenRepo");
+        downloadOK = false;
+      }
+      String fpDownloaded = fDownloaded.getAbsolutePath();
+      fpDownloaded = fpDownloaded.replace("-forsetup", "");
+      fDownloaded.renameTo(new File(fpDownloaded));
+    }
+
     if (getIDE || getAPI) {
       if (forSystemLux || forAllSystems) {
         jarsList[8] = new File(workDir, libsLux + ".jar").getAbsolutePath();
@@ -690,11 +721,20 @@ public class RunSetup {
           fDownloaded = downloadJarFromMavenSx(libsLux, dlDir, libsLux);
         }
         downloadOK &= copyFromDownloads(fDownloaded, libsLux, jarsList[8]);
-        if (isLinux) {
+        if (downloadOK && isLinux) {
           runTime.addToClasspath(jarsList[8]);
           runTime.dumpClassPath("sikulix");
-          logPlus(lvl, "checking usability of bundled/provided libs");
-          RunTime.loadLibrary(LinuxSupport.slibVision, useLibsProvided);
+          if (shouldBuildVision) {
+            logPlus(lvl, "Requested to build libVisionProxy.so on the fly");
+            if (!LinuxSupport.haveToBuild()) {
+              terminate("Building libVisionproxy.so not possible - check the log");
+            }
+          }
+          logPlus(lvl, "checking usability of bundled, provided or built libs");
+          if (!RunTime.loadLibrary(LinuxSupport.slibVision, useLibsProvided)) {
+            logPlus(-1, "libVisionproxy.so finally not useable");
+            terminate("Giving up!");
+          } else logPlus(lvl, "Bundled, provided or built libVisionproxy.so is useable");
           useLibsProvided = runTime.useLibsProvided || LinuxSupport.shouldUseProvided;
         }
       }
@@ -727,7 +767,7 @@ public class RunSetup {
         downloadOK &= copyFromDownloads(fDownloaded, libsMac, jarsList[7]);
       }
     }
-		//</editor-fold>
+    //</editor-fold>
 
     //<editor-fold defaultstate="collapsed" desc="download IDE/API Jython/JRuby ...">
     if (getIDE || getAPI) {
@@ -738,7 +778,7 @@ public class RunSetup {
         fDownloaded = downloadJarFromMavenSx("sikulixsetupAPI#forsetup", dlDir, sDownloaded);
       }
       boolean dlApiOK = copyFromDownloads(fDownloaded, sDownloaded, localJar);
-      if(dlApiOK && (forSystemWin || forAllSystems)) {
+      if (dlApiOK && (forSystemWin || forAllSystems)) {
         FileManager.resetFolder(folderLib);
         String aJar = FileManager.normalizeAbsolute(localJar, false);
         if (null == runTime.resourceListAsSikulixContentFromJar(aJar, "Lib", folderLib, null)) {
@@ -772,12 +812,13 @@ public class RunSetup {
         downloadedFiles.replace(downloadJython, downloadJython25);
       } else {
         if (popAsk("If you click YES, you will get Jython version 2.7.0 (recommended)\n"
-                + "... but there is an issue with some rare UTF-8/Unicode situations,\n"
-                + "that usually appear on startup with UTF-8 ccharacters\n"
-                + "somewhere in the system environment\n"
+                + "... but in rare cases there might be issues with UTF-8/Unicode\n"
+                + "that usually appear on startup when UTF-8 characters\n"
+                + "are present somewhere in the system environment\n"
                 + "If you encounter such problems with Jython 2.7.0\n"
                 + "run setup again and\n"
                 + "click NO to get Jython 2.5.4rc1")) {
+          // TODO: use runtime.SikuliJythonVersion25 in the message above
           sDownloadedName = new File(runTime.SikuliJythonMaven).getName();
           fDownloaded = downloadedAlready("python", "Jython 2.7", false);
           if (fDownloaded == null) {
@@ -800,7 +841,7 @@ public class RunSetup {
       targetJar = new File(workDir, localJRuby).getAbsolutePath();
       fDownloaded = downloadsFound.get("ruby");
       if (fDownloaded == null) {
-          fDownloaded = downloadJarFromMaven(runTime.SikuliJRubyMaven, dlDirGeneric, sDownloaded);
+        fDownloaded = downloadJarFromMaven(runTime.SikuliJRubyMaven, dlDirGeneric, sDownloaded);
       }
       downloadOK &= copyFromDownloads(fDownloaded, sDownloaded, targetJar);
       if (downloadOK && getJRubyAddOns) {
@@ -825,7 +866,7 @@ public class RunSetup {
         }
       }
     }
-		//</editor-fold>
+    //</editor-fold>
 
     //<editor-fold defaultstate="collapsed" desc="download for tesseract">
     if (getTess) {
@@ -862,8 +903,8 @@ public class RunSetup {
         targetJar = fTargetJar.getAbsolutePath();
         String tessJar = new File(workDir, localTess).getAbsolutePath();
 
-				success = runTime.addToClasspath(fTessData.getParent());
-				runTime.resourceListAsSikulixContent(tessFolder, fTessData, null);
+        success = runTime.addToClasspath(fTessData.getParent());
+        runTime.resourceListAsSikulixContent(tessFolder, fTessData, null);
 
         downloadOK &= FileManager.buildJar("#" + targetJar, new String[]{},
                 new String[]{fTessData.getAbsolutePath()},
@@ -880,12 +921,11 @@ public class RunSetup {
       logPlus(lvl, "Downloads for selected options:\n" + downloadedFiles);
     }
     if (!downloadOK) {
-      String msg = "Some of the downloads did not complete successfully.\n"
+      msg = "Some of the downloads did not complete successfully.\n"
               + "Check the logfile for possible error causes.\n\n"
               + "If you think, setup's inline download is blocked somehow on\n"
               + "your system, you might download the appropriate raw packages manually\n"
               + "into the folder Downloads in the setup folder and run setup again.\n\n"
-              + "download page: " + runTime.downloadBaseDirWeb + "\n"
               + "files to download (information is in the setup log file too)\n\n";
       for (String fnToDownload : downloadedFiles.split(" ")) {
         msg += fnToDownload + "\n";
@@ -976,7 +1016,7 @@ public class RunSetup {
 
     splash = showSplash("Now creating jars, application and commandfiles", "please wait - may take some seconds ...");
 
-    File fAPI = new File(workDir, localAPI); 
+    File fAPI = new File(workDir, localAPI);
     jarsList[1] = fAPI.getAbsolutePath();
     hasAPI = fAPI.exists();
 
@@ -993,7 +1033,7 @@ public class RunSetup {
       success &= handleTempAfter(targetJar, localJar);
     }
 
-    if(getAPI && getTess) {
+    if (getAPI && getTess) {
       new File(workDir, localTess).delete();
       jarsList[2] = null;
     }
@@ -1034,8 +1074,6 @@ public class RunSetup {
       }
     }
 
-    closeSplash(splash);
-
     if (success && getIDE) {
       logPlus(lvl, "processing commandfiles");
       if (runTime.runningWindows) {
@@ -1048,8 +1086,10 @@ public class RunSetup {
         new File(fWorkDir, runsikulix).setExecutable(true);
         new File(fWorkDir, localIDE).setExecutable(true);
       }
-      closeSplash(splash);
     }
+
+    closeSplash(splash);
+
     if (!success) {
       popError("Bad things happened trying to add native stuff to selected jars --- terminating!");
       terminate("Adding stuff to jars did not work");
@@ -1185,14 +1225,24 @@ public class RunSetup {
       closeSplash(splash);
     }
 
-    log(lvl,
-            "... SikuliX Setup seems to have ended successfully ;-)");
-    
+    logPlus(lvl, "... SikuliX Setup seems to have ended successfully ;-)");
+
     finalCleanup();
 
     System.exit(RunTime.testing ? 1 : 0);
   }
-  
+
+  private static String arrayToString(String[] args) {
+    String ret = "";
+    for (String s : args) {
+      if (s.contains(" ")) {
+        s = "\"" + s + "\"";
+      }
+      ret += s + " ";
+    }
+    return ret;
+  }
+
   private static void finalCleanup() {
     if (hasAPI) jarsList[1] = null;
     for (int i = (getAPI ? 2 : 1); i < jarsList.length; i++) {
@@ -1247,21 +1297,22 @@ public class RunSetup {
     downloadsLookfor.put("tess", "tesseract");
     downloadsFound.put("tess", null);
 
-    String doubleFiles = "";     
-    for (File aFolder : new File[] {
-      fWorkDir, fDownloadsObsolete, fDownloadsGenericApp, fDownloadsGeneric}) {
+    String doubleFiles = "";
+    for (File aFolder : new File[]{
+            fWorkDir, fDownloadsObsolete, fDownloadsGenericApp, fDownloadsGeneric}) {
       File[] filesContained = aFolder.listFiles(new FilenameFilter() {
-          List<String> valid = new ArrayList<String>(downloadsLookfor.values());
-          @Override
-          public boolean accept(File dir, String name) {
-            for (String sFile : valid) {
-              if (name.startsWith(sFile)) {
-                return true;
-              }
+        List<String> valid = new ArrayList<>(downloadsLookfor.values());
+
+        @Override
+        public boolean accept(File dir, String name) {
+          for (String sFile : valid) {
+            if (name.startsWith(sFile)) {
+              return true;
             }
-            return false;
           }
-        });
+          return false;
+        }
+      });
       if (filesContained != null) {
         for (File aFile : filesContained) {
           for (String prefix : downloadsLookfor.keySet()) {
@@ -1289,7 +1340,7 @@ public class RunSetup {
       } else {
         log(lvl, "checkDownloads: not found: %s", prefix);
       }
- }
+    }
     if (!doubleFiles.isEmpty()) {
       popError("The following files are double or even more often found in the\n"
               + "respective folders setup checks before downloading new artefacts:\n" + doubleFiles +
@@ -1298,75 +1349,75 @@ public class RunSetup {
       terminate("double downloaded files");
     }
   }
-  
+
   private static boolean createSetupFolder(File fTargetDir) {
-		String projectDir = runTime.fSxProject.getAbsolutePath();
-		boolean success = true;
+    String projectDir = runTime.fSxProject.getAbsolutePath();
+    boolean success = true;
 
-		File fSetup = getProjectJarFile(projectDir, "Setup", "sikulixsetup", "-forsetup.jar");
-		success &= fSetup != null;
-		File fIDEPlus = getProjectJarFile(projectDir, "SetupIDE", "sikulixsetupIDE", "-forsetup.jar");
-		success &= fIDEPlus != null;
-		File fAPIPlus = getProjectJarFile(projectDir, "SetupAPI", "sikulixsetupAPI", "-forsetup.jar");
-		success &= fAPIPlus != null;
-		File fLibsmac = getProjectJarFile(projectDir, "Libsmac", libsMac, ".jar");
-		success &= fLibsmac != null;
-		File fLibswin = getProjectJarFile(projectDir, "Libswin", libsWin, ".jar");
-		success &= fLibswin != null;
-		File fLibslux = getProjectJarFile(projectDir, "Libslux", libsLux, ".jar");
-		success &= fLibslux != null;
+    File fSetup = getProjectJarFile(projectDir, "Setup", "sikulixsetup", "-forsetup.jar");
+    success &= fSetup != null;
+    File fIDEPlus = getProjectJarFile(projectDir, "SetupIDE", "sikulixsetupIDE", "-forsetup.jar");
+    success &= fIDEPlus != null;
+    File fAPIPlus = getProjectJarFile(projectDir, "SetupAPI", "sikulixsetupAPI", "-forsetup.jar");
+    success &= fAPIPlus != null;
+    File fLibsmac = getProjectJarFile(projectDir, "Libsmac", libsMac, ".jar");
+    success &= fLibsmac != null;
+    File fLibswin = getProjectJarFile(projectDir, "Libswin", libsWin, ".jar");
+    success &= fLibswin != null;
+    File fLibslux = getProjectJarFile(projectDir, "Libslux", libsLux, ".jar");
+    success &= fLibslux != null;
 
-		File fJythonJar = new File(runTime.SikuliJython);
-		if (!noSetup && !fJythonJar.exists()) {
-			log(lvl, "createSetupFolder: missing: " + fJythonJar.getAbsolutePath());
-			success = false;
-		}
-		File fJythonJar25 = new File(runTime.SikuliJython25);
-		if (!noSetup && !fJythonJar25.exists()) {
-			log(lvl, "createSetupFolder: missing: " + fJythonJar.getAbsolutePath());
+    File fJythonJar = new File(runTime.SikuliJython);
+    if (!noSetup && !fJythonJar.exists()) {
+      log(lvl, "createSetupFolder: missing: " + fJythonJar.getAbsolutePath());
+      success = false;
+    }
+    File fJythonJar25 = new File(runTime.SikuliJython25);
+    if (!noSetup && !fJythonJar25.exists()) {
+      log(lvl, "createSetupFolder: missing: " + fJythonJar.getAbsolutePath());
       fJythonJar25 = null;
-		}
-		File fJrubyJar = new File(runTime.SikuliJRuby);
-		if (!noSetup && !fJrubyJar.exists()) {
-			log(lvl, "createSetupFolder: missing " + fJrubyJar.getAbsolutePath());
-			success = false;
-		}
+    }
+    File fJrubyJar = new File(runTime.SikuliJRuby);
+    if (!noSetup && !fJrubyJar.exists()) {
+      log(lvl, "createSetupFolder: missing " + fJrubyJar.getAbsolutePath());
+      success = false;
+    }
 
-		if (success) {
-			FileManager.resetFolder(fDownloadsGenericApp);
-			success &= FileManager.xcopy(fSetup, new File(fTargetDir, localSetup));
+    if (success) {
+      FileManager.resetFolder(fDownloadsGenericApp);
+      success &= FileManager.xcopy(fSetup, new File(fTargetDir, localSetup));
       if (success) {
         for (String sFile : fTargetDir.list()) {
-          if (sFile.contains("sikulixsetup") && 
+          if (sFile.contains("sikulixsetup") &&
                   sFile.contains("-project") &&
                   !sFile.contains(localSetup)) {
             FileManager.deleteFileOrFolder(new File(fTargetDir, sFile));
           }
         }
       }
-			success &= FileManager.xcopy(fIDEPlus, new File(fDownloadsGenericApp, downloadIDE));
-			success &= FileManager.xcopy(fAPIPlus, new File(fDownloadsGenericApp, downloadAPI));
+      success &= FileManager.xcopy(fIDEPlus, new File(fDownloadsGenericApp, downloadIDE));
+      success &= FileManager.xcopy(fAPIPlus, new File(fDownloadsGenericApp, downloadAPI));
 
-			for (File fEntry : new File[]{fLibsmac, fLibswin, fLibslux}) {
-				success &= FileManager.xcopy(fEntry, new File(fDownloadsGenericApp, fEntry.getName()));
-			}
+      for (File fEntry : new File[]{fLibsmac, fLibswin, fLibslux}) {
+        success &= FileManager.xcopy(fEntry, new File(fDownloadsGenericApp, fEntry.getName()));
+      }
 
-			if (!noSetup) {
-				success &= FileManager.xcopy(fJythonJar, new File(fDownloadsGeneric, downloadJython));
+      if (!noSetup) {
+        success &= FileManager.xcopy(fJythonJar, new File(fDownloadsGeneric, downloadJython));
         if (fJythonJar25 != null) {
           FileManager.xcopy(fJythonJar25, new File(fDownloadsGeneric, downloadJython25));
         }
-				success &= FileManager.xcopy(fJrubyJar, new File(fDownloadsGeneric, downloadJRuby));
-			}
+        success &= FileManager.xcopy(fJrubyJar, new File(fDownloadsGeneric, downloadJRuby));
+      }
 
 //TODO JRubyAddOns
-			String jrubyAddons = "sikulixjrubyaddons-" + runTime.SikuliProjectVersion + "-plain.jar";
-			File fJRubyAddOns = new File(projectDir, "JRubyAddOns/target/" + jrubyAddons);
+      String jrubyAddons = "sikulixjrubyaddons-" + runTime.SikuliProjectVersion + "-plain.jar";
+      File fJRubyAddOns = new File(projectDir, "JRubyAddOns/target/" + jrubyAddons);
 //        success &= FileManager.xcopy(fJRubyAddOns, new File(fDownloadsGeneric, downloadJRubyAddOns));
-      
-		}
-		return success;
-	}
+
+    }
+    return success;
+  }
 
   private static File getProjectJarFile(String project, String jarFileDir, String jarFilePre, String jarFileSuf) {
     String jarFileName = getProjectJarFileName(jarFilePre, jarFileSuf);
@@ -1552,14 +1603,14 @@ public class RunSetup {
   private static void popError(String msg) {
     logPlus(3, "\npopError: " + packMessage(msg));
     if (!hasOptions) {
-      Sikulix.popError(msg, "SikuliX-Setup: having problems ...");
+      JOptionPane.showMessageDialog(null, msg, "SikuliX-Setup: having problems ...", JOptionPane.ERROR_MESSAGE);
     }
   }
 
   private static void popInfo(String msg) {
     logPlus(3, "\npopInfo: " + packMessage(msg));
     if (!hasOptions) {
-      Sikulix.popup(msg, "SikuliX-Setup: info ...");
+      JOptionPane.showMessageDialog(null, msg, "SikuliX-Setup: info ...", JOptionPane.PLAIN_MESSAGE);
     }
   }
 
@@ -1568,11 +1619,15 @@ public class RunSetup {
     if (hasOptions) {
       return true;
     }
-    return Sikulix.popAsk(msg, "SikuliX-Setup: question ...");
+    int ret = JOptionPane.showConfirmDialog(null, msg, "SikuliX-Setup: question ...", JOptionPane.YES_NO_OPTION);
+    if (ret == JOptionPane.CLOSED_OPTION || ret == JOptionPane.NO_OPTION) {
+      return false;
+    }
+    return true;
   }
 
   private static JFrame showSplash(String title, String msg) {
-    if (hasOptions) {
+    if (hasOptions | notests) {
       return null;
     }
     start = (new Date()).getTime();
@@ -1623,7 +1678,7 @@ public class RunSetup {
     return new File(fname);
   }
 
-//libDownloaded = takeAlreadyDownloaded(RunFolder, RunFolder/Downloads/ dlDirBuild, dlDirGeneric, libsLux);
+  //libDownloaded = takeAlreadyDownloaded(RunFolder, RunFolder/Downloads/ dlDirBuild, dlDirGeneric, libsLux);
   private static File downloadedAlready(String item, String itemName, boolean isVersioned) {
     File targetFolder = isVersioned ? fDownloadsGenericApp : fDownloadsGeneric;
     File artefact = downloadsFound.get(item);
@@ -1649,7 +1704,7 @@ public class RunSetup {
       if (runningWithProject) {
         return artefact;
       }
-      if (popAsk("You have for "+ itemName + "\n"
+      if (popAsk("You have for " + itemName + "\n"
               + artefact.getAbsolutePath()
               + "\nClick YES, if you want to use this for setup processing\n\n"
               + "... or click NO, to ignore it and download a fresh copy")) {
@@ -1684,11 +1739,11 @@ public class RunSetup {
       itemSuffix = "-" + parts[1];
     }
     if (runTime.isVersionRelease()) {
-      if (itemSuffix.contains("forsetup") || item.contains("sikulixlibs")) {
-        mPath = runTime.downloadBaseDir;
-      } else {
-        mPath = String.format("%s%s/%s/", sikulixMavenGroup, item, version);
-      }
+//      if (itemSuffix.contains("forsetup") || item.contains("sikulixlibs")) {
+//        mPath = runTime.downloadBaseDir;
+//      } else {
+      mPath = String.format("%s%s/%s/", sikulixMavenGroup, item, version);
+//      }
       mJar = String.format("%s-%s%s.jar", item, version, itemSuffix);
     } else {
       String dlMavenSnapshotPath = version + "-SNAPSHOT";
@@ -1747,7 +1802,16 @@ public class RunSetup {
     if (item.startsWith("http")) {
       return download(item, target, null, itemName);
     } else {
-      return download(runTime.dlMavenRelease + item, target, null, itemName);
+      if (useLocalMavenRepo) {
+        try {
+          URL theFile = new URL("file", null, runTime.SikuliLocalRepo + item);
+          return download(theFile.toExternalForm(), target, null, itemName);
+        } catch (MalformedURLException e) {
+          return null;
+        }
+      } else {
+        return download(runTime.dlMavenRelease + item, target, null, itemName);
+      }
     }
   }
 
